//------------------------------------------------------------------------------
// application.cpp
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "application.h"

#include <QDebug>

#include "keyboard/keystatemonitor.h"
#include "keyboard/keystatereader.h"
#include "keyboard/shortcutsmonitor.h"

const int Application::KEY_STATE_READ_INTERVAL_MS = 25;

Application::Application(int& argc, char** argv)
: QApplication(argc, argv, true)
{
	pKeyStateMonitor = new KeyStateMonitor();
	pShortcutsMonitor = new ShortcutsMonitor(pKeyStateMonitor);
	pKeyStateReader = KeyStateReader::instantiateOsSpecificReader();

	if (pKeyStateReader != NULL)
	{
		pKeyStateMonitor->setKeys(pKeyStateReader->allKnownQtKeys());
		pKeyStateMonitor->startMonitoring(pKeyStateReader, KEY_STATE_READ_INTERVAL_MS);
	}
	else
	{
		qDebug() << "Warning: KeyStateReader is not supported on this OS. "
			<< "The program will not detect keyboard shortcuts.";
	}
}

Application::~Application()
{
	pKeyStateMonitor->stopMonitoring();
	if (pKeyStateReader != NULL)
	{
		delete pKeyStateReader;
	}

	delete pShortcutsMonitor;
	delete pKeyStateMonitor;
}

KeyStateMonitor* Application::keyStateMonitor()
{
	return pKeyStateMonitor;
}

KeyStateReader* Application::keyStateReader()
{
	return pKeyStateReader;
}

Application* Application::instance()
{
	return (Application*)qApp;
}

void Application::registerShortcut(Shortcut* pShortcut)
{
	pShortcutsMonitor->registerShortcut(pShortcut);
}

ShortcutsMonitor* Application::shortcutsMonitor()
{
	return pShortcutsMonitor;
}
