//------------------------------------------------------------------------------
// audiocore.cpp
//
// Copyright (C) 2011 - 2012 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "audiocore.h"

#include <QList>

const QString AudioCore::AUDIO_PCM_CODEC = "audio/pcm";

QAudioDeviceInfo AudioCore::audioDeviceInfoFromDevName(QAudio::Mode mode, const QString& name)
{
	fixAudioDeviceList();

	QList<QAudioDeviceInfo> availableDevices = QAudioDeviceInfo::availableDevices(mode);
	foreach (const QAudioDeviceInfo& info, availableDevices)
	{
		if (info.deviceName().compare(name, Qt::CaseInsensitive) == 0)
		{
			return info;
		}
	}
	
	return QAudioDeviceInfo();
}

QString AudioCore::audioFormatEndianEnumToString(QAudioFormat::Endian val)
{
	switch (val)
	{
		case QAudioFormat::BigEndian:
			return "Big endian";
			
		case QAudioFormat::LittleEndian:
			return "Little endian";
			
		default:
			return "Unknown:QAudioFormat::Endian";
	}
}

QString AudioCore::audioFormatSampleTypeEnumToString(QAudioFormat::SampleType val)
{
	switch (val)
	{
		case QAudioFormat::Float:
			return "Float";
			
		case QAudioFormat::SignedInt:
			return "Signed int";
			
		case QAudioFormat::Unknown:
			return "Unknown";
		
		case QAudioFormat::UnSignedInt:
			return "Unsigned int";
			
		default:
			return "Unknown:QAudioFormat::SampleType";
	}
}

void AudioCore::fixAudioDeviceList()
{
	// Because Qt.
	QAudioDeviceInfo::defaultInputDevice();
	QAudioDeviceInfo::defaultOutputDevice();
}

bool AudioCore::hasAnyAudioDevices(QAudio::Mode mode)
{
	fixAudioDeviceList();
	QList<QAudioDeviceInfo> availableDevices = QAudioDeviceInfo::availableDevices(mode);
	
	return !availableDevices.isEmpty();
}
