//------------------------------------------------------------------------------
// audiocore.h
//
// Copyright (C) 2011 - 2012 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#ifndef __AUDIOCORE_H__
#define __AUDIOCORE_H__

#include <QAudioDeviceInfo>
#include <QString>

/**
 * @brief Provides general convenient access methods to QtMultimedia API.
 */
class AudioCore
{
	public:
		/**
		 * @brief Default audio codec that should be used in the application.
		 *
		 * Value: "audio/pcm"
		 */
		static const QString AUDIO_PCM_CODEC;
	
		/**
		 * @brief Gets QAudioDeviceInfo where its deviceName() fits
		 *        the specified one.
		 *
		 * @param mode
		 *      Input or Output. See Qt docs.
		 * @param name
		 *      Name of the device. Case-insensitive.
		 * @return QAudioDeviceInfo ready to use in other Qt classes.
		 *         If audio device cannot be obtained, then 
		 *         QAudioDeviceInfo::isNull() will return <code>true</code>
		 */
		static QAudioDeviceInfo audioDeviceInfoFromDevName(QAudio::Mode mode, const QString& name);
		
		static QString audioFormatEndianEnumToString(QAudioFormat::Endian val);
		static QString audioFormatSampleTypeEnumToString(QAudioFormat::SampleType val);
		
		static bool hasAnyAudioDevices(QAudio::Mode mode);
		
	private:
		static void fixAudioDeviceList();
};

#endif
