//------------------------------------------------------------------------------
// audioinputformatfactory.cpp
//
// Copyright (C) 2012 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "audioinputformatfactory.h"

#include "audio/audiocore.h"
#include "config/audioinputconfig.h"
#include <QList>

AudioInputFormatFactory::AudioInputFormatFactory()
{
	pCfg = NULL;
}

AudioInputFormatFactory::~AudioInputFormatFactory()
{
}

QAudioDeviceInfo AudioInputFormatFactory::getAudioDeviceInfo(const QString& deviceName)
{
	return AudioCore::audioDeviceInfoFromDevName(QAudio::AudioInput, deviceName);
}

QAudioDeviceInfo AudioInputFormatFactory::getAudioDeviceInfo()
{
	if (pCfg != NULL)
	{
		QAudioDeviceInfo dev = getAudioDeviceInfo(pCfg->deviceName());
		if (!dev.isNull())
		{
			return dev;
		}
	}
	return QAudioDeviceInfo::defaultInputDevice();
}

QAudioFormat AudioInputFormatFactory::getAudioFormat(const QAudioDeviceInfo& dev)
{
	QAudioFormat calculatedFormat = getPreferredFormat(dev);
	
	// This is hard-coded because WE NEED the raw PCM data to write the
	// .wav file.
	calculatedFormat.setCodec(AudioCore::AUDIO_PCM_CODEC);
	
	if (pCfg != NULL)
	{
		if (pCfg->isSetByteOrder())
		{
			calculatedFormat.setByteOrder(pCfg->byteOrder());
		}
		
		if (pCfg->isSetChannelCount())
		{
			calculatedFormat.setChannelCount(pCfg->channelCount());
		}
		
		if (pCfg->isSetSampleRate())
		{
			calculatedFormat.setSampleRate(pCfg->sampleRate());
		}
		
		if (pCfg->isSetSampleSize())
		{
			calculatedFormat.setSampleSize(pCfg->sampleSize());
		}
		
		if (pCfg->isSetSampleType())
		{
			calculatedFormat.setSampleType(pCfg->sampleType());
		}
	}
	
	return dev.nearestFormat(calculatedFormat);
}

QAudioFormat AudioInputFormatFactory::getPreferredFormat(const QAudioDeviceInfo& dev) const
{
	QAudioFormat preferredFormat = dev.preferredFormat();
	
	// This needs to be set. This REALLY needs to be set.
	// Qt said that preferred format for my mic is 11025.
	if (preferredFormat.sampleRate() < PREFERRED_SAMPLE_RATE)
	{
		preferredFormat.setSampleRate(PREFERRED_SAMPLE_RATE);
	}
	
	// This needs to be set. This REALLY needs to be set.
	// Another "Qt preferredFormat()" failure.
	if (preferredFormat.sampleSize() < PREFERRED_SAMPLE_SIZE)
	{
		preferredFormat.setSampleSize(PREFERRED_SAMPLE_SIZE);
	}
	
	preferredFormat.setChannelCount(PREFERRED_CHANNEL_COUNT);
	
	return preferredFormat;
}
