//------------------------------------------------------------------------------
// audioinputformatfactory.h
//
// Copyright (C) 2012 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#ifndef __AUDIOINPUTFORMATFACTORY_H__
#define __AUDIOINPUTFORMATFACTORY_H__

#include <QAudioDeviceInfo>
#include <QAudioFormat>

class AudioInputConfig;

class AudioInputFormatFactory
{
	public:
		/**
		 * @brief Get QAudioDeviceInfo where device name fits the specified one.
		 *
		 * Check is case-insensitive.
		 *
		 * @return A valid QAudioDeviceInfo if device is found. 
		 *         In case of failure isNull() method of returned object will
		 *         return false.
		 */
		static QAudioDeviceInfo getAudioDeviceInfo(const QString& deviceName);
	
		AudioInputFormatFactory();
		~AudioInputFormatFactory();

		/**
		 * @brief Cfg stored device info or default one.
		 */
		QAudioDeviceInfo getAudioDeviceInfo();
		
		QAudioFormat getAudioFormat(const QAudioDeviceInfo& dev);
		
		void setConfig(AudioInputConfig* pCfg)
		{
			this->pCfg = pCfg;
		}
		
	private:
		/**
		 * @brief Set to 1 because stereo microphones are rarely used
		 *        for amateur tasks.
		 */
		static const int PREFERRED_CHANNEL_COUNT = 1;
		static const int PREFERRED_SAMPLE_RATE = 44100;
		static const int PREFERRED_SAMPLE_SIZE = 16;
	
		AudioInputConfig* pCfg;
		
		QAudioFormat getPreferredFormat(const QAudioDeviceInfo& dev) const;
};

#endif