//------------------------------------------------------------------------------
// wavcontainer.h
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#ifndef __WAVCONTAINER_H__
#define __WAVCONTAINER_H__

#include <QAudioFormat>
#include <QFile>
#include <QIODevice>

class WAVContainer : public QIODevice
{
	public:
		WAVContainer(QIODevice* pIODevice = NULL, QObject* pParent = NULL);
		~WAVContainer();
		
		const QAudioFormat& audioFormat() const
		{
			return d.audioFormat;
		}
		
		void close(); 
		bool open(QIODevice::OpenMode mode);

		/**
		 * @brief Reads and stores the WAV header in QAudioFormat.
		 *
		 * Read WAV header information can be accessed through
		 * audioFormat() method.
		 *
		 * @return true if read was successful
		 */
		bool readWAVHeader();
		
		bool reset();
		bool seek(qint64 pos);
		void setAudioFormat(const QAudioFormat& format);
		
		/**
		 * @brief Sets IO device that will provide access to the WAV data.
		 *
		 * @b Note:
		 * The WAVContainer doesn't take ownership of the passed QIODevice.
		 * This device needs to be deleted, opened and closed on its own.
		 */
		void setIODevice(QIODevice* pDevice);
		
		qint64 size() const;
		
		bool writeDataSize();
		
		/**
		 * @brief Writes the RIFF WAVE header.
		 *
		 * The header is generated basing on the current audio format.
		 * This will overwrite the previous header contents if they are 
		 * encountered. 
		 *
		 * Once the header is written the cursor of the underlying QIODevice
		 * object will be set to the position of first byte of data.
		 *
		 * In other words: calling this function is an easy way to reset
		 * the entire file.
		 */
		bool writeWAVHeader();
		
	protected:
		qint64 readData(char* data, qint64 maxSize);
		qint64 writeData(const char* data, qint64 maxSize);
		
	private:	
		class PrivData
		{
			public:
				QAudioFormat audioFormat;
				QIODevice* pIODevice;
				quint64 dataPos;
		};
		
		PrivData d;
		
		qint64 readDataSize(QIODevice& dev);
		QAudioFormat readWAVHeader(QIODevice& dev);		
		bool writeDataSize(QIODevice& dev, qint64 dataSize);
		bool writeWAVHeader(QIODevice& dev, const QAudioFormat& format);
};

#endif
