//------------------------------------------------------------------------------
// amplifyfilter.cpp
//
// Copyright (C) 2013 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "amplifyfilter.h"

#include <QDataStream>
#include <QDebug>
#include <cmath>
#include <qmath.h>
#include "audio/filters/filtermetadata.h"
#include "config/filters/amplifyfiltercfg.h"
#include "gui/filterconfig/amplifyfiltercfgbox.h"

class AmplifyFilter::PrivData
{
	public:
		double amplifyDecibel;
};
//////////////////////////////////////////////////////////////////////////////
AmplifyFilter::AmplifyFilter(const FilterMetaData& metaData)
: AudioFilter(metaData)
{
	d = new PrivData();
	AmplifyFilterCfg cfg(gpConfig);
	d->amplifyDecibel = cfg.powerLevel();
}

AmplifyFilter::~AmplifyFilter()
{
	delete d;
}

QWidget* AmplifyFilter::configWidget(QWidget* parentWidget)
{
	return new AmplifyFilterCfgBox(parentWidget);
}

bool AmplifyFilter::feed(QIODevice& in, QIODevice& out)
{
	QDataStream::ByteOrder byteOrder =
		static_cast<QDataStream::ByteOrder>(metaData().audioFormat.byteOrder());
	QDataStream inStream(&in);
	inStream.setByteOrder(byteOrder);

	QDataStream outStream(&out);
	outStream.setByteOrder(byteOrder);

	while (!in.atEnd())
	{
		qint32 sample = readSample(inStream);
		qint32 sampleAmplified = qPow(10, log10(qAbs((double)sample)) + (d->amplifyDecibel / 20.0));
		if (sample < 0)
		{
			// Minus sign is lost in previous calculation.
			sampleAmplified = -sampleAmplified;
		}
		writeSample(outStream, sampleAmplified);
	}
	return true;
}

qint32 AmplifyFilter::readSample(QDataStream& stream)
{
	switch (metaData().audioFormat.sampleSize())
	{
		case 8:
		{
			// For some reason the recorded samples are unsigned bytes, even
			// if audio format states otherwise. It must be tested whether
			// this is true on all systems or just on Windows.
			quint8 sample;
			stream >> sample;
			return (qint32)(sample - 0x80);
		}
		case 16:
		{
			qint16 sample;
			stream >> sample;
			return (qint32)sample;
		}
		default:
		{
			qint32 sample;
			stream >> sample;
			return sample;
		}
	}
}

void AmplifyFilter::writeSample(QDataStream& stream, qint32 sample)
{
	switch (metaData().audioFormat.sampleSize())
	{
		case 8:
			// For some reason the recorded samples are unsigned bytes, even
			// if audio format states otherwise. It must be tested whether
			// this is true on all systems or just on Windows.
			stream << (quint8)qBound(0x0, sample + 0x80, 0xff);
			break;
		case 16:
			stream << (qint16)qBound(-0x8000, sample, 0x7fff);
			break;
		default:
			stream << sample;
			break;
	}
}
