//------------------------------------------------------------------------------
// filterpipeline.cpp
//
// Copyright (C) 2013 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "filterpipeline.h"

#include "audio/filters/audiofilter.h"
#include <QBuffer>
#include <QDebug>
#include <cassert>

class FilterPipeline::PrivData
{
	public:
		QList<AudioFilter*> filters;

		QBuffer outBuffer;
};
//////////////////////////////////////////////////////////////////////////////
FilterPipeline::FilterPipeline()
{
	d = new PrivData();
	d->outBuffer.open(QIODevice::ReadWrite);
}

FilterPipeline::~FilterPipeline()
{
	delete d;
}

void FilterPipeline::addFilter(AudioFilter* filter)
{
	d->filters.append(filter);
}

const QList<AudioFilter*>& FilterPipeline::filters() const
{
	return d->filters;
}

bool FilterPipeline::open(QIODevice::OpenMode mode)
{
	if (mode != QIODevice::ReadWrite)
	{
		qDebug() << "Attempted to open FilterPipeline for something else than ReadWrite.";
		assert(false && "Attempted to open FilterPipeline for something else than ReadWrite.");
		return false;
	}
	return QIODevice::open(mode);
}

qint64 FilterPipeline::readData(char* data, qint64 maxSize)
{
	d->outBuffer.seek(0);
	qint64 readSize = d->outBuffer.read(data, maxSize);
	resetQBuffer(d->outBuffer, d->outBuffer.data().mid(readSize));
	return readSize;
}

void FilterPipeline::resetQBuffer(QBuffer& buffer, const QByteArray& data)
{
	buffer.close();
	buffer.setData(data);
	buffer.open(QIODevice::ReadWrite);
}

qint64 FilterPipeline::writeData(const char* data, qint64 maxSize)
{
	QBuffer inBuffer;
	QBuffer outBuffer;
	inBuffer.setData(data, maxSize);
	inBuffer.open(QIODevice::ReadWrite);
	outBuffer.open(QIODevice::ReadWrite);
	foreach (AudioFilter* filter, d->filters)
	{
		if (!filter->feed(inBuffer, outBuffer))
		{
			return -1;
		}
		resetQBuffer(inBuffer, outBuffer.data());
		resetQBuffer(outBuffer);
	}
	d->outBuffer.seek(d->outBuffer.size());
	qint64 writtenSize = d->outBuffer.write(inBuffer.readAll());
	emit readyRead();
	return writtenSize;
}
