//------------------------------------------------------------------------------
// audioinputconfig.cpp
//
// Copyright (C) 2011 - 2012 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "audioinputconfig.h"

#include "audio/audiocore.h"

#include <QString>

const QString SECTION_NAME = "AudioInput";

const QString DATA_BYTE_ORDER = "DataByteOrder";
const QString CHANNEL_COUNT = "ChannelCount";
const QString CODEC = "Codec";
const QString DEVICE_NAME = "DeviceName";
const QString SAMPLE_RATE = "SampleRate";
const QString SAMPLE_SIZE = "SampleSize";
const QString SAMPLE_TYPE = "SampleType";

AudioInputConfig::AudioInputConfig(Config* pConfig)
: ConfigSection(pConfig, SECTION_NAME)
{
}

bool AudioInputConfig::isSetByteOrder() const
{
	return value(DATA_BYTE_ORDER).isValid();
}

bool AudioInputConfig::isSetChannelCount() const
{
	return value(CHANNEL_COUNT).isValid();
}

bool AudioInputConfig::isSetCodec() const
{
	return value(CODEC).isValid();
}

bool AudioInputConfig::isSetDeviceName() const
{
	return !deviceName().isEmpty();
}

bool AudioInputConfig::isSetSampleRate() const
{
	return value(SAMPLE_RATE).isValid();
}

bool AudioInputConfig::isSetSampleSize() const
{
	return value(SAMPLE_SIZE).isValid();
}

bool AudioInputConfig::isSetSampleType() const
{
	return value(SAMPLE_TYPE).isValid();
}

QAudioFormat::Endian AudioInputConfig::byteOrder() const
{
	return (QAudioFormat::Endian) value(DATA_BYTE_ORDER, QAudioFormat::LittleEndian).toInt();
}

int AudioInputConfig::channelCount() const
{
	return value(CHANNEL_COUNT, 1).toInt();
}

QString AudioInputConfig::codec() const
{
	return value(CODEC, AudioCore::AUDIO_PCM_CODEC).toString();
}

QString AudioInputConfig::deviceName() const
{
	return value(DEVICE_NAME, "").toString();
}

int AudioInputConfig::sampleRate() const
{
	return value(SAMPLE_RATE, 44100).toInt();
}

int AudioInputConfig::sampleSize() const
{
	return value(SAMPLE_SIZE, 8).toInt();
}

QAudioFormat::SampleType AudioInputConfig::sampleType() const
{
	return (QAudioFormat::SampleType) value(SAMPLE_TYPE, QAudioFormat::Unknown).toInt();
}

void AudioInputConfig::setByteOrder(QAudioFormat::Endian endian)
{
	setValue(DATA_BYTE_ORDER, QAudioFormat::Endian(QSysInfo::ByteOrder));
}

void AudioInputConfig::setChannelCount(int count)
{
	setValue(CHANNEL_COUNT, count);
}

void AudioInputConfig::setCodec(const QString& str)
{
	setValue(CODEC, str);
}

void AudioInputConfig::setDeviceName(const QString& str)
{
	setValue(DEVICE_NAME, str);
}

void AudioInputConfig::setSampleRate(int sampleRate)
{
	setValue(SAMPLE_RATE, sampleRate);
}

void AudioInputConfig::setSampleSize(int sampleSize)
{
	setValue(SAMPLE_SIZE, sampleSize);
}

void AudioInputConfig::setSampleType(QAudioFormat::SampleType sampleType)
{
	setValue(SAMPLE_TYPE, sampleType);
}
