//------------------------------------------------------------------------------
// config.cpp
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "config.h"

#include "versioninfo.h"
#include <cassert>

Config* Config::pConfig = NULL;

Config::Config()
{
	bIsFirstRun = false;
	pSettings = NULL;
}

Config::~Config()
{
	if (pSettings != NULL)
	{
		delete pSettings;
	}
}

void Config::deinit()
{
	if (pConfig != NULL)
	{
		delete pConfig;
	}
}

void Config::init(const QString& filename)
{
	const QString FIRST_RUN_VARNAME = "HelloWorld/HelloWorld";
	const int FIRST_RUN_EXPECTED_VALUE = 1;

	// Create the CFG file access interface.
	Config* pConfig = new Config();
	pConfig->pSettings = new QSettings(QSettings::IniFormat, QSettings::UserScope, 
		VersionInfo::appDataDirName(), VersionInfo::mainConfigFileName());
	
	// Check if this is the first run on this CFG.
	QVariant firstRunVal = pConfig->pSettings->value(FIRST_RUN_VARNAME, -1);
	pConfig->bIsFirstRun = (firstRunVal.toInt() != FIRST_RUN_EXPECTED_VALUE);

	// Update the FIRST_RUN information.
	pConfig->pSettings->setValue(FIRST_RUN_VARNAME, 
		FIRST_RUN_EXPECTED_VALUE);
	
	// Store the config in the static field for singleton.
	Config::pConfig = pConfig;
}

QSettings& Config::settings()
{
	assert(pSettings != NULL);
	return *pSettings;
}

void Config::synchronize()
{
	assert(pSettings != NULL);
	pSettings->sync();
}
