//------------------------------------------------------------------------------
// configsection.h
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#ifndef __CONFIGSECTION_H__
#define __CONFIGSECTION_H__

#include "config/config.h"
#include <QString>

class ConfigSection
{
	public:
		ConfigSection(Config* pConfig, const QString& name);
		
	protected:
		QString sectionName;
		mutable Config* pConfig;
		
		/**
		 * @brief Sets value in the section.
		 *
		 * @param varName
		 *      Variable name within the section. This is passed to var() and
		 *      the var() generated key name is used instead.
		 * @param value
		 *      A QVariant value that will be set.
		 */
		void setValue(const QString& varName, const QVariant& value);
		
		/**
		 * @brief Retrieves value from the section.
		 *
		 * @param varName
		 *      Variable name within the section. This is passed to var() and
		 *      the var() generated key name is used instead.
		 * @param defaultValue
		 *      Optional parameter. Value that is returned if varName is not found.
		 * @return QVariant retrieved from the specified key name.
		 */
		QVariant value(const QString& varName, const QVariant& defaultValue = QVariant()) const;
		
		/**
		 * @brief Generates full variable name basing on the section's name
		 *        and passed parameter.
		 *
		 * @return <code>sectionName + "/" + name</code> or just 
		 *         <code>name</code> if sectionName is empty.
		 *
		 */
		QString var(const QString& name) const;
};

#endif