//------------------------------------------------------------------------------
// generalconfig.cpp
//
// Copyright (C) 2011 - 2012 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "generalconfig.h"

#include <QDesktopServices>
#include <QFileInfo>
#include "paths.h"

const QString SECTION_NAME = "General";

const QString IS_CLOSE_TO_TRAY_ICON_ENABLED = "bIsCloseToTrayIconEnabled";
const QString IS_MINIMIZE_TO_TRAY_ICON_ENABLED = "bIsMinimizeToTrayIconEnabled";
const QString IS_TRAY_ICON_ENABLED = "bIsTrayIconEnabled";
const QString OUTPUT_FILE = "OutputFile";

const QString OUTPUT_FILE_DEFAULT = Paths::documentsPath() + "$TIMESTAMP.wav";

GeneralConfig::GeneralConfig(Config* pConfig)
: ConfigSection(pConfig, SECTION_NAME)
{
}

bool GeneralConfig::isCloseToTrayIconEnabled() const
{
	return value(IS_CLOSE_TO_TRAY_ICON_ENABLED, false).toBool();
}

bool GeneralConfig::isMinimizeToTrayIconEnabled() const
{
	return value(IS_MINIMIZE_TO_TRAY_ICON_ENABLED, false).toBool();
}

bool GeneralConfig::isTrayIconEnabled() const
{
	return value(IS_TRAY_ICON_ENABLED, false).toBool();
}
		
QString GeneralConfig::outputFile() const
{
	return value(OUTPUT_FILE, OUTPUT_FILE_DEFAULT).toString();
}

void GeneralConfig::setCloseToTrayIconEnabled(bool b)
{
	setValue(IS_CLOSE_TO_TRAY_ICON_ENABLED, b);
}

void GeneralConfig::setOutputFile(const QString& fileName)
{
	setValue(OUTPUT_FILE, fileName);
}

void GeneralConfig::setMinimizeToTrayIconEnabled(bool b)
{
	setValue(IS_MINIMIZE_TO_TRAY_ICON_ENABLED, b);
}

void GeneralConfig::setTrayIconEnabled(bool b)
{
	setValue(IS_TRAY_ICON_ENABLED, b);
}
