//------------------------------------------------------------------------------
// notificationsconfig.cpp
//
// Copyright (C) 2012 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "notificationsconfig.h"

#include <QCoreApplication>
#include <QFileInfo>
#include "paths.h"

const QString SECTION_NAME = "Notifications";

const QString IS_SHOW_RECORDING_TIME_ON_TITLE_BAR_ENABLED = "IsShowRecordingTimeOnTitleBarEnabled";
const QString IS_STARTED_RECORDING_SOUND_ENABLED = "IsStartedRecordingSoundEnabled";
const QString IS_STOPPED_RECORDING_SOUND_ENABLED = "IsStoppedRecordingSoundEnabled";
const QString STARTED_RECORDING_SOUND_PATH = "StartedRecordingSoundPath";
const QString STOPPED_RECORDING_SOUND_PATH = "StoppedRecordingSoundPath";

const QString DEFAULT_STARTED_RECORDING_SUBPATH = "sounds/started_rec.wav";
const QString DEFAULT_STOPPED_RECORDING_SUBPATH = "sounds/stopped_rec.wav";

NotificationsConfig::NotificationsConfig(Config* pConfig)
: ConfigSection(pConfig, SECTION_NAME)
{
}

bool NotificationsConfig::isShowRecordingTimeOnTitleBarEnabled() const
{
	return value(IS_SHOW_RECORDING_TIME_ON_TITLE_BAR_ENABLED, false).toBool();
}

bool NotificationsConfig::isStartedRecordingSoundEnabled() const
{
	return value(IS_STARTED_RECORDING_SOUND_ENABLED, false).toBool();
}

bool NotificationsConfig::isStoppedRecordingSoundEnabled() const
{
	return value(IS_STOPPED_RECORDING_SOUND_ENABLED, false).toBool();
}

QString NotificationsConfig::startedRecordingSoundPath() const
{
	QString path = QCoreApplication::applicationDirPath() + "/" 
		+ DEFAULT_STARTED_RECORDING_SUBPATH;
		
	return value(STARTED_RECORDING_SOUND_PATH, path).toString();
}

QString NotificationsConfig::stoppedRecordingSoundPath() const
{
	QString path = QCoreApplication::applicationDirPath() + "/" 
		+ DEFAULT_STOPPED_RECORDING_SUBPATH;
		
	return value(STOPPED_RECORDING_SOUND_PATH, path).toString();
}

void NotificationsConfig::setIsShowRecordingTimeOnTitleBarEnabled(bool b)
{
	setValue(IS_SHOW_RECORDING_TIME_ON_TITLE_BAR_ENABLED, b);
}

void NotificationsConfig::setIsStartedRecordingSoundEnabled(bool b)
{
	setValue(IS_STARTED_RECORDING_SOUND_ENABLED, b);
}

void NotificationsConfig::setIsStoppedRecordingSoundEnabled(bool b)
{
	setValue(IS_STOPPED_RECORDING_SOUND_ENABLED, b);
}

void NotificationsConfig::setStartedRecordingSoundPath(const QString& val)
{
	setValue(STARTED_RECORDING_SOUND_PATH, val);
}

void NotificationsConfig::setStoppedRecordingSoundPath(const QString& val)
{
	setValue(STOPPED_RECORDING_SOUND_PATH, val);
}
