//------------------------------------------------------------------------------
// configdialog.cpp
//
// Copyright (C) 2011 - 2012 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "configdialog.h"

#include <QDebug>
#include <QMessageBox>

ConfigDialog::ConfigDialog(QWidget* pParent)
: QDialog(pParent)
{
	setupUi(this);
	
	// Updates the first tab.
	tabChanged(0);
}

void ConfigDialog::accept()
{
	if (saveConfig())
	{
		done(QDialog::Accepted);
	}
}

void ConfigDialog::apply()
{
	saveConfig();
}

void ConfigDialog::buttonClicked(QAbstractButton* pButton)
{
	if (pButton == (QAbstractButton*)buttonBox->button(QDialogButtonBox::Apply))
	{
		apply();
	}
}

bool ConfigDialog::saveConfig()
{
	ConfigTab* tabs[] =
	{
		tabGeneral,
		tabAudio,
		tabNotifications,
		tabKeyShortcuts,
		tabFilters,
		NULL
	};

	QList<ConfigTab*> tabsToSave;
	for (int i = 0; tabs[i] != NULL; ++i)
	{
		if (tabs[i]->wasShown())
		{
			if (!validateConfigTab(tabs[i]))
			{
				tabWidget->setCurrentWidget(tabs[i]);
				return false;
			}
			tabsToSave << tabs[i];	
		}
	}

	for (int i = 0; i < tabsToSave.size(); ++i)
	{
		qDebug() << "Saving config: " << tabsToSave[i]->name();
		tabsToSave[i]->saveConfig();
	}

	return true;
}

void ConfigDialog::tabChanged(int index)
{
	ConfigTab* pTab = (ConfigTab*)tabWidget->currentWidget();
	
	if (pTab != NULL)
	{
		pTab->tabSelected();
	}
}

bool ConfigDialog::validateConfigTab(ConfigTab* pTab)
{
	QString error = pTab->validate();
	if (error.isEmpty())
	{
		return true;
	}
	else
	{
		QMessageBox::critical(this, tr("Configuration validation error"), error);
		return false;
	}
}
