//------------------------------------------------------------------------------
// configtab.h
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#ifndef __CONFIGTAB_H__
#define __CONFIGTAB_H__

#include <QLabel>
#include <QString>
#include <QWidget>

class IconLabel;

class ConfigTab : public QWidget
{
	public:
		ConfigTab(QWidget* pParent = NULL);
		virtual ~ConfigTab() {}
		
		virtual QString name() const = 0;
		
		virtual void loadConfig() = 0;
		virtual void saveConfig() = 0;
		
		/**
		 * @brief Called when tab is selected.
		 *
		 * @b Note:
		 * It is important to call this function when reimplementing. 
		 * The default behavior calls tabSelectedFirstTime if necessary.
		 */
		virtual void tabSelected();
		
		/**
		 * @brief Calls loadConfig().
		 */
		virtual void tabSelectedFirstTime();

		/**
		 * @brief Validates the config.
		 *
		 * Default behavior does nothing and returns an empty string.
		 *
		 * @return Empty string if validation passed, error message if
		 *         validation failed.
		 */
		virtual QString validate()
		{
			return QString();
		};
		
		/**
		 * @brief True if tab was shown at least once.
		 */
		bool wasShown() const
		{
			return bFirstShowDone;
		}
		
	protected:
		enum ErrorLabelType
		{
			/**
			 * @brief Displays red exclamation mark.
			 */
			ErrorLabel,
			
			/**
			 * @brief Displays yellow exclamation mark in a triangle.
			 */
			WarningLabel
		};
	
		/**
		 * @brief Shows specified label with given icon and text.
		 *
		 * This should be used to display the validation failure labels.
		 *
		 * @param pLabel
		 *      QLabel widget to show.
		 * @param type
		 *      Type of error. See: ErrorLabelType
		 * @param text
		 *      Text that will be shown in the label widget.
		 */
		void showErrorLabel(IconLabel* pLabel, ErrorLabelType type, const QString& text);
		
	private:
		bool bFirstShowDone;
};

#endif
