//------------------------------------------------------------------------------
// filterscfgtab.cpp
//
// Copyright (C) 2013 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "gui/configtabs/filterscfgtab.h"

#include "audio/filters/audiofilter.h"
#include "audio/filters/filterfactory.h"
#include "audio/filters/filtermetadata.h"
#include <QDebug>
#include <QDialog>
#include <QListWidgetItem>
#include <QMessageBox>
#include <QScopedPointer>

FiltersCfgTab::FiltersCfgTab(QWidget* pParent)
: ConfigTab(pParent)
{
	setupUi(this);
	btnConfigure->setEnabled(false);
}

QStringList FiltersCfgTab::allActiveFiltersNames() const
{
	QStringList result;
	for (int i = 0; i < lvActiveFilters->count(); ++i)
	{
		result << lvActiveFilters->item(i)->data(Qt::UserRole).toString();
	}
	return result;
}

void FiltersCfgTab::applyConfigureButtonEnabledState()
{
	btnConfigure->setEnabled(canEnableConfigureButton());
}

bool FiltersCfgTab::canEnableConfigureButton() const
{
	QStringList filters = selectedActiveFiltersNames();
	if (filters.size() == 1)
	{
		QScopedPointer<AudioFilter> filter(FilterFactory::construct(filters[0],
			FilterMetaData()));
		return filter->isConfigurable();
	}
	return false;
}

void FiltersCfgTab::configureDoubleClickedFilter(QListWidgetItem* item)
{
	configureFilter(item->data(Qt::UserRole).toString());
}

void FiltersCfgTab::configureFilter(const QString& filterName)
{
	QScopedPointer<AudioFilter> filter(FilterFactory::construct(filterName,
			FilterMetaData()));
	if (filter != NULL && filter->isConfigurable())
	{
		QScopedPointer<QDialog> configBox((QDialog*)filter->configWidget(this));
		if (configBox == NULL)
		{
			QMessageBox::warning(this, tr("Configure Filter"),
				tr("Failed to create config box for selected filter. "
					"This is a bug in the program."));
			return;
		}
		configBox->exec();
	}
	else
	{
		QMessageBox::warning(this, tr("Configure Filter"),
			tr("Selected filter is not configurable."));
	}
}

void FiltersCfgTab::configureSelectedActiveFilter()
{
	QStringList filters = selectedActiveFiltersNames();
	if (filters.size() != 1)
	{
		QMessageBox::warning(this, tr("Configure Filter"),
			tr("Select exactly one active filter."));
		return;
	}
	configureFilter(filters[0]);
}

void FiltersCfgTab::loadConfig()
{
	FiltersConfig cfg(gpConfig);
	QStringList enabledFilters = cfg.enabledFilters();
	QMap<QString, QString> allFiltersNames = FilterFactory::allFilterNames();
	
	foreach (const QString& filter, allFiltersNames)
	{
		QListWidgetItem* item = new QListWidgetItem();
		item->setText(allFiltersNames[filter]);
		item->setData(Qt::UserRole, filter);
		if (enabledFilters.contains(filter))
		{
			lvActiveFilters->addItem(item);
		}
		else
		{
			lvAvailableFilters->addItem(item);
		}
	}
}

void FiltersCfgTab::saveConfig()
{
	FiltersConfig cfg(gpConfig);
	cfg.setEnabledFilters(allActiveFiltersNames());
}

QStringList FiltersCfgTab::selectedActiveFiltersNames() const
{
	QStringList result;
	foreach (QListWidgetItem* item, lvActiveFilters->selectedItems())
	{
		result << item->data(Qt::UserRole).toString();
	}
	return result;
}

QString FiltersCfgTab::validate()
{
	return QString();
}
