//------------------------------------------------------------------------------
// generalcfgtab.cpp
//
// Copyright (C) 2011 - 2012 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "gui/configtabs/generalcfgtab.h"

#include "config/generalconfig.h"

#include <QDebug>
#include <QDir>
#include <QFileDialog>
#include <QFileInfo>

GeneralCfgTab::GeneralCfgTab(QWidget* pParent)
: ConfigTab(pParent)
{
	setupUi(this);

	hideErrorLabels();
}

void GeneralCfgTab::hideErrorLabels()
{
	lblOutputFileError->hide();
}

void GeneralCfgTab::loadConfig()
{
	GeneralConfig cfg(gpConfig);
	
	grbUseTrayIcon->setChecked(cfg.isTrayIconEnabled());
	cbCloseToTrayIcon->setChecked(cfg.isCloseToTrayIconEnabled());
	cbMinimizeToTrayIcon->setChecked(cfg.isMinimizeToTrayIconEnabled());
	leOutputFile->setText(cfg.outputFile());
}

void GeneralCfgTab::saveConfig()
{
	GeneralConfig generalCfg(gpConfig);
	
	generalCfg.setTrayIconEnabled(grbUseTrayIcon->isChecked());
	generalCfg.setCloseToTrayIconEnabled(cbCloseToTrayIcon->isChecked());
	generalCfg.setMinimizeToTrayIconEnabled(cbMinimizeToTrayIcon->isChecked());
	generalCfg.setOutputFile(leOutputFile->text());
}

void GeneralCfgTab::showPickOutputPathDialog()
{
	QString currentPath = leOutputFile->text();
	QFileInfo fileInfo(currentPath);
	const QString& dirPath = fileInfo.absolutePath();

	QString newPath = QFileDialog::getExistingDirectory(this, 
		tr("Pick output directory"), dirPath,
		QFileDialog::ShowDirsOnly | QFileDialog::ReadOnly);
		
	if (!newPath.isNull())
	{
		const QString& fileNameTemplate = fileInfo.fileName();
		newPath = newPath.trimmed();
		newPath += "/" + fileNameTemplate;
		newPath = QDir::cleanPath(newPath);
		leOutputFile->setText(newPath);
	}
}

QString GeneralCfgTab::validate()
{
	hideErrorLabels();

	return validateOutputPath();
}

QString GeneralCfgTab::validateOutputPath()
{
	QString path = leOutputFile->text();
	path = path.trimmed();
	if (path.isEmpty())
	{
		showErrorLabel(lblOutputFileError, ErrorLabel, tr("Path must be specified."));
		return tr("Output path is not specified.");
	}

	QFileInfo fileInfo(path);
	QDir dir(fileInfo.absolutePath());
	qDebug() << "Checking if " << fileInfo.absolutePath() << " exists";
	if (!dir.exists())
	{
		showErrorLabel(lblOutputFileError, ErrorLabel, tr("Directory doesn't exist."));
		return tr("Directory in the output path doesn't exist.");
	}
	
	if (fileInfo.isDir())
	{
		showErrorLabel(lblOutputFileError, ErrorLabel, tr("This is a directory."));
		return tr("Specified output path exists but points to a directory!");
	}

	return QString();
}
