//------------------------------------------------------------------------------
// notificationscfgtab.cpp
//
// Copyright (C) 2012 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "gui/configtabs/notificationscfgtab.h"

#include "config/notificationsconfig.h"

#include <QDebug>
#include <QDir>
#include <QFileDialog>
#include <QFileInfo>
#include <QMessageBox>
#include <QSound>

NotificationsCfgTab::NotificationsCfgTab(QWidget* pParent)
: ConfigTab(pParent)
{
	setupUi(this);

	hideErrorLabels();
}

void NotificationsCfgTab::hideErrorLabels()
{
	lblError->hide();
}

void NotificationsCfgTab::loadConfig()
{
	NotificationsConfig cfg(gpConfig);
	
	cbShowRecordingTimeOnTitleBar->setChecked(cfg.isShowRecordingTimeOnTitleBarEnabled());
	
	cbStartedRecording->setChecked(cfg.isStartedRecordingSoundEnabled());
	cbStoppedRecording->setChecked(cfg.isStoppedRecordingSoundEnabled());
	leStartedRecording->setText(cfg.startedRecordingSoundPath());
	leStoppedRecording->setText(cfg.stoppedRecordingSoundPath());
}

void NotificationsCfgTab::playSoundIfExistsOrDisplayError(const QString& path)
{
	QFileInfo fileInfo(path);
	
	if (fileInfo.isFile())
	{
		QSound::play(path);
	}
	else
	{
		if (!fileInfo.exists())
		{
			QMessageBox::critical(this, tr("Failed to play sound"),
				tr("Specified file does not exist."));
		}
		else if (!fileInfo.isFile())
		{
			QMessageBox::critical(this, tr("Failed to play sound"),
				tr("Specified path does not point to a file."));
		}
	}
}

void NotificationsCfgTab::playStartedRecordingSound()
{
	playSoundIfExistsOrDisplayError(leStartedRecording->text());
}

void NotificationsCfgTab::playStoppedRecordingSound()
{
	playSoundIfExistsOrDisplayError(leStoppedRecording->text());
}

void NotificationsCfgTab::saveConfig()
{
	NotificationsConfig cfg(gpConfig);
	
	cfg.setIsShowRecordingTimeOnTitleBarEnabled(cbShowRecordingTimeOnTitleBar->isChecked());
	
	cfg.setIsStartedRecordingSoundEnabled(cbStartedRecording->isChecked());
	cfg.setIsStoppedRecordingSoundEnabled(cbStoppedRecording->isChecked());
	cfg.setStartedRecordingSoundPath(leStartedRecording->text());
	cfg.setStoppedRecordingSoundPath(leStoppedRecording->text());
}

void NotificationsCfgTab::showPickStartedRecordingPathDialog()
{
	QString currentPath = leStartedRecording->text();
	QString newPath = showPickWavFileDialog(currentPath);
			
	if (!newPath.isNull())
	{
		leStartedRecording->setText(newPath);
	}
}

void NotificationsCfgTab::showPickStoppedRecordingPathDialog()
{
	QString currentPath = leStoppedRecording->text();
	QString newPath = showPickWavFileDialog(currentPath);
			
	if (!newPath.isNull())
	{
		leStoppedRecording->setText(newPath);
	}
}


QString NotificationsCfgTab::showPickWavFileDialog(const QString& defaultPath)
{
	QString dirPath;
	if (!defaultPath.trimmed().isEmpty())
	{
		QFileInfo fileInfo(defaultPath);
		QString dirPath = fileInfo.absolutePath();
	}

	QString newPath = QFileDialog::getOpenFileName(this, 
		tr("Pick sound file"), dirPath,
		tr("WAVE (*.wav)"));
		
	if (!newPath.isNull())
	{
		newPath = newPath.trimmed();
		newPath = QDir::cleanPath(newPath);
	}
	
	return newPath;
}

QString NotificationsCfgTab::validate()
{
	hideErrorLabels();

	if (cbStartedRecording->isChecked())
	{
		QString result = validateFilePath(leStartedRecording->text());
		if (!result.isEmpty())
		{
			result = tr("Started recording: %1").arg(result);
			showErrorLabel(lblError, ErrorLabel, result);
			return result;
		}
	}
	
	if (cbStoppedRecording->isChecked())
	{
		QString result = validateFilePath(leStoppedRecording->text());
		if (!result.isEmpty())
		{
			result = tr("Stopped recording: %1").arg(result);
			showErrorLabel(lblError, ErrorLabel, result);
			return result;
		}
	}
	
	return QString();
}

QString NotificationsCfgTab::validateFilePath(const QString& path)
{
	QString trimmedPath = path.trimmed();
	if (trimmedPath.isEmpty())
	{
		return tr("Path is not specified.");
	}

	QFileInfo fileInfo(trimmedPath);
	qDebug() << "Checking if" << fileInfo.absoluteFilePath() << "exists";
	if (!fileInfo.exists())
	{
		return tr("File doesn't exist.");
	}
	
	if (!fileInfo.isFile())
	{
		return tr("Path exists but is not a file!");
	}

	return QString();
}
