//------------------------------------------------------------------------------
// mainwindow.h
//
// Copyright (C) 2011 - 2012 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#ifndef __MAINWINDOW_H__
#define __MAINWINDOW_H__

#include "ui_mainwindow.h"
#include <QString>
#include "keyboard/keyinfo.h"

#define gpMainWindow MainWindow::instance()

class Shortcut;
class TrayIcon;

class MainWindow : public QMainWindow, private Ui::MainWindow
{
	Q_OBJECT

	public:
		static void destroy()
		{
			if (pInstance != NULL)
			{
				delete pInstance;
				pInstance = NULL;
			}
		}

		static void init(QWidget* pParent = NULL)
		{
			if (pInstance == NULL)
			{
				pInstance = new MainWindow(pParent);
			}
		}

		static MainWindow* instance()
		{
			return pInstance;
		}

		/**
		 *	@brief Returns true if window was maximized before it was
		 *	minimized.
		 */
		bool wasMaximized() const
		{
			return bWasMaximized;
		}

	public slots:
		/**
		 *	@brief Adjusts height to fit proportionally to the
		 *	screen on which the program currently exists.
		 *
		 *	This method will make sure that only the available
		 *	desktop space is used, ie. the taskbar is not taken
		 *	into consideration.
		 */
		void adjustHeight();

		/**
		 *	@brief Shows the window at the top.
		 *
		 *	If necessary calls either showNormal() or showMaximized()
		 *	depending on bWasMaximized flag and then calls activateWindow().
		 */
		void bringUp();

		/**
		 *	@brief Centers window on the screen
		 *	on which the program currently exists.
		 *
		 *	This method will make sure that only the available
		 *	desktop space is used, ie. the taskbar is not taken
		 *	into consideration.
		 */
		void centerOnDesktop();
		
		/**
		 * @brief Closes the window disregarding if "close to tray"
		 *        option is enabled or not.
		 */
		void closeForce();

	protected:
		bool event(QEvent* pEvent);

	private:
		class TrayIconMenu : public QMenu
		{
			public:
				QAction* pActionShow;
				QAction* pActionExit;
		
				TrayIconMenu(QWidget* pParent);
				~TrayIconMenu();
		};
	
		static MainWindow* pInstance;

		bool bForceClosing;
		bool bWasMaximized;
		bool bWasShown;
		Shortcut* pShortcutStartRecording;
		Shortcut* pShortcutStopRecording;
		Shortcut* pShortcutToggleRecording;
		TrayIcon* pTrayIcon;
		TrayIconMenu* pTrayIconMenu;

		MainWindow(QWidget* pParent = NULL);
		~MainWindow();

		void applyShortcutsConfiguration();
		
		/**
		 * @brief Checks if it is OK to close the window to the tray
		 *        icon.
		 *
		 * @attention
		 * The close 'X' button default behavior will be overridden and
		 * it will not quit the program anymore.
		 */
		bool canCloseToTrayIcon() const;
		
		/**
		 * @brief Checks if it is OK to minimize the window to the tray
		 *        icon.
		 */
		bool canMinimizeToTrayIcon() const;
		
		void doConnections();
		
		/**
		 * @brief Checks if tray icon is not NULL and is visible.
		 */
		bool isTrayIconVisible() const;
		
		void setTitleToDefault();
		void setTrayIconAppearanceAccordingToState();
		void setTrayIconVisibilityAccordingToConfig();
		void setupShortcuts();
		void setupTrayIcon();

	private slots:
		void browseOutputDir();
	
		/**
		 * This is a slot because it needs to be called from a timer.
		 */
		void firstShow();

		void keyStateChanged(KeyInfo::KeyId key, int state);
		void recordingNotify();
		void recordingStateChanged(bool bIsRecording);
		void showAboutDialog();
		void showAboutQtDialog();
		void showConfigurationDialog();
		void trayIconMenuActivated(QAction* pAction);
};

#endif
