//------------------------------------------------------------------------------
// recordarea.h
//
// Copyright (C) 2011 - 2012 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#ifndef __RECORDAREA_H__
#define __RECORDAREA_H__

#include "ui_recordarea.h"

#include <QAudioFormat>
#include <QAudioInput>
#include <QIODevice>
#include <QWidget>

class WAVContainer;

class RecordArea : public QWidget, private Ui::RecordArea
{
	Q_OBJECT;

	public:
		RecordArea(QWidget* pParent = NULL);
		~RecordArea();
		
		bool isRecording() const;
		
		/**
		 * @brief Path to the newest created file.
		 *
		 * This is updated each time a new recording is started and
		 * will remain the same even after the recording is stopped.
		 *
		 * This may be empty if no recording was started yet.
		 */
		const QString& newestRecordedFilePath() const;
		
		/**
		 * @brief Format of the newest created file.
		 *
		 * This is updated each time a new recording is started and
		 * will remain the same even after the recording is stopped.
		 *
		 * This may be invalid if no recording was started yet.
		 */
		const QAudioFormat& newestRecordingFormat() const;
		
		qint64 recordingTimeMs() const;
		
	public slots:
		void startRecording();
		void stopRecording();
		void toggleRecording();
		
	signals:
		/**
		 * @brief Emitted for every QAudioInput::notify()
		 */
		void recordingNotify();
		void recordingStateChanged(bool bIsRecording);
		
	private:
		class PrivData;

		PrivData* d;

		QString generateOutputPath();
		void setupRecorder();
		void switchStateToRecording();
		void switchStateToStopped();

	private slots:
		void audioInputStateChanged(QAudio::State state);
		void audioInputNotify();
};

#endif