//------------------------------------------------------------------------------
// shortcutcreatorlineedit.cpp
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "shortcutcreatorlineedit.h"

#include <QDebug>
#include <QKeyEvent>
#include "keyboard/keydecoder.h"
#include "keyboard/keystatemonitor.h"
#include "application.h"

ShortcutCreatorLineEdit::ShortcutCreatorLineEdit(QWidget* pParent)
: QLineEdit(pParent)
{
	keyPressCount = 0;
	
	this->connect(gpApp->keyStateMonitor(),
		SIGNAL(keyStateChanged(KeyInfo::KeyId, int)),
		SLOT  (keyStateChanged(KeyInfo::KeyId, int)) );
}

void ShortcutCreatorLineEdit::addKey(KeyInfo::KeyId key)
{	
	QString codeName = KeyDecoder::toCodeName(key);

	currentSequence << codeName;
	currentSequence.removeDuplicates();

	setText(currentSequence.join("+"));
}

void ShortcutCreatorLineEdit::clear()
{
	keyPressCount = 0;
	QLineEdit::clear();
	currentSequence.clear();
}

bool ShortcutCreatorLineEdit::event(QEvent* pEvent)
{
	if (pEvent->type() == QEvent::KeyPress)
	{
		// We don't want this to be handled. The keydown/keyup
		// events are handled through the KeyStateMonitor.
		// See: keyStateChanged()
		return true;
	}
	else if (pEvent->type() == QEvent::FocusIn)
	{
		keyPressCount = 0;
	}

	return QLineEdit::event(pEvent);
}

QList<KeyInfo::KeyId> ShortcutCreatorLineEdit::keys() const
{
	return KeyDecoder::parseSequence(text());
}

void ShortcutCreatorLineEdit::keyStateChanged(KeyInfo::KeyId key, int state)
{
	if (hasFocus())
	{
		if (state == KeyStateMonitor::Pressed)
		{
			if (keyPressCount <= 0)
			{
				clear();
			}
			
			++keyPressCount;	
			addKey(key);
		}
		else if (state == KeyStateMonitor::Depressed)
		{
			--keyPressCount;
		}
	}
}

void ShortcutCreatorLineEdit::setKeys(const QList<KeyInfo::KeyId>& keys)
{
	setText(KeyDecoder::toSequence(keys));
}
