//------------------------------------------------------------------------------
// shortcutcreatorlineedit.h
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#ifndef __SHORTCUTCREATORLINEEDIT_H__
#define __SHORTCUTCREATORLINEEDIT_H__

#include "keyboard/keyinfo.h"

#include <QEvent>
#include <QKeySequence>
#include <QLineEdit>

/**
 * @brief A subclass of QLineEdit that accepts keystrokes and displays them
 *        ready to use in QKeySequence.
 *
 * @b Note:
 * This doesn't support the numpad, yet. Qt has no built-in support for this.
 * This is a problem that will be hard to resolve while keeping platform
 * independency.
 */
class ShortcutCreatorLineEdit : public QLineEdit
{
	Q_OBJECT;

	public:
		ShortcutCreatorLineEdit(QWidget* pParent = NULL);
		virtual ~ShortcutCreatorLineEdit() {}
		
		void addKey(KeyInfo::KeyId key);
		QList<KeyInfo::KeyId> keys() const;
		void setKeys(const QList<KeyInfo::KeyId>& keys);

	public slots:
		void clear();

	protected:
		virtual bool event(QEvent* pEvent);

	private:
		QStringList currentSequence;
		int keyPressCount;
		
	private slots:
		void keyStateChanged(KeyInfo::KeyId key, int state);
};

#endif
