//------------------------------------------------------------------------------
// timelabel.cpp
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "timelabel.h"

#include <QDateTime>
#include <QDebug>
#include <cassert>

TimeLabel::TimeLabel(QWidget* pParent)
: QLabel(pParent)
{
	d.autoplaySpeedFactor = 1.0;
	d.time = 0LL;
	updateDisplay();
	
	this->connect(&d.autoplayTimer, SIGNAL( timeout() ),
		SLOT( autoplayIncrease() ) );
}

void TimeLabel::autoplayIncrease()
{
	qint64 elapsed = d.lastAutoplayTimestamp.elapsed();
	d.lastAutoplayTimestamp.start(); 
	
	increase(elapsed);
}

void TimeLabel::increase(qint64 msec)
{
	d.time += msec;
	updateDisplay();
}

void TimeLabel::setTime(qint64 msec)
{
	d.time = msec;
	updateDisplay();
}

void TimeLabel::startAutoplay()
{
	startAutoplay(1.0);
}

void TimeLabel::startAutoplay(double speedFactor)
{
	assert(speedFactor > 0.0);	
	
	if (speedFactor <= 0.0)
	{
		return;
	}
	
	d.autoplaySpeedFactor = speedFactor;
	d.lastAutoplayTimestamp.start();
	
	d.autoplayTimer.start(1000 * speedFactor);
}

void TimeLabel::stopAutoplay()
{
	d.autoplayTimer.stop();
}

qint64 TimeLabel::time() const
{
	return d.time;
}

void TimeLabel::updateDisplay()
{
	qint64 msecs = d.time;
	qint64 secs = msecs / 1000;
	qint64 mins = secs / 60;
	qint64 hours = mins / 60;
	
	secs %= 60;
	mins %= 60;
	
	QString strTime = QString("%1:%2:%3")
		.arg(hours, 2, 10, QLatin1Char('0'))
		.arg(mins, 2, 10, QLatin1Char('0'))
		.arg(secs, 2, 10, QLatin1Char('0'));
		
	setText(strTime);
}
