//------------------------------------------------------------------------------
// keyboardhook.cpp
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "keyboardhook.h"

#include <cstdlib>
#include <QDebug>

KeyboardHook* KeyboardHook::pInstance = NULL;

KeyboardHook::KeyboardHook()
{

}

void KeyboardHook::install()
{
	qDebug() << "Installing keyboard hook";
	installHookOsSpecific();
}

void KeyboardHook::uninstall()
{
	qDebug() << "Removing keyboard hook";
	uninstallHookOsSpecific();
}

//////////////////////////////////////////////////////////////////////////////
// System specific actions.
#ifdef Q_WS_WIN
#include <Windows.h>
HHOOK hook = NULL;

LRESULT CALLBACK keyboardProcedure(int nCode, WPARAM wParam, LPARAM lParam)
{
	// Check for a key down press
	if (nCode == HC_ACTION)
	{
		if (wParam == WM_KEYDOWN)
		{
			KeyboardHook::instance()->emitKeyDownDetected();
		}
		else if (wParam == WM_KEYUP)
		{
			KeyboardHook::instance()->emitKeyUpDetected();
		}

	}

	return CallNextHookEx(hook, nCode, wParam, lParam);
}

void KeyboardHook::installHookOsSpecific()
{
	hook = SetWindowsHookEx(WH_KEYBOARD_LL, keyboardProcedure, GetModuleHandle(NULL), 0);
}

void KeyboardHook::uninstallHookOsSpecific()
{
	UnhookWindowsHookEx(hook);
}
#else
void KeyboardHook::installHookOsSpecific()
{
	qDebug() << "KeyboardHook::installHookOsSpecific() is not supported on current OS";
}

void KeyboardHook::uninstallHookOsSpecific()
{
	qDebug() << "KeyboardHook::uninstallHookOsSpecific() is not supported on current OS";
}
#endif
