//------------------------------------------------------------------------------
// keyinfo.cpp
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "keyinfo.h"

#include <QDebug>
#include <cassert>

bool KeyInfo::bIsInit = false;
KeyInfo KeyInfo::keys[NUM_KEYS];

KeyInfo::KeyInfo()
{
	d.asciiCode = 0;
	d.keyNum = 0;
}

KeyInfo::KeyInfo(int asciiCode, int keyNum, const QString& codeName, const QString& niceName)
{
	d.asciiCode = asciiCode;
	d.codeName = codeName;
	d.keyNum = keyNum;
	d.niceName = niceName;
}

KeyInfo::KeyInfo(int asciiCode, int keyNum, const QString& codeName)
{
	d.asciiCode = asciiCode;
	d.codeName = codeName;
	d.keyNum = keyNum;
	d.niceName = codeName;
}

bool KeyInfo::assertKeyCodes()
{
	for (int i = 0; i < NUM_KEYS; ++i)
	{
		if (i != keys[i].keyId())
		{
			qDebug() << "[" << i << "]: Assertion fail, invalid keyId() "
				<< keys[i].keyId() << "key:" << keys[i].codeName();
			return false;
		}
	}
	
	return true;
}

void KeyInfo::initKeys()
{
	for (int key = Key_0; key <= Key_9; ++key)
	{
		int i = key - Key_0;
		char c = (char)0x30 + i;
		QString name = QString(c);
		keys[key] = KeyInfo(c, key, name, name);
	}

	for (int key = Key_A; key <= Key_Z; ++key)
	{
		int i = key - Key_A;
		char c = (char)0x41 + i;
		QString name = QString(c);
		keys[key] = KeyInfo(c, key, name, name);
	}

	keys[BackSlash] = KeyInfo('\\', BackSlash, "BackSlash", "BackSlash");
	keys[Slash] = KeyInfo('/', Slash, "Slash", "Slash");
	keys[Semicolon] = KeyInfo(';', Semicolon, "Semicolon");
	keys[LBracket] = KeyInfo('{', LBracket, "LBracket", "Left Bracket");
	keys[RBracket] = KeyInfo('}', RBracket, "RBracket", "Right Bracket");
	keys[Comma] = KeyInfo(',', Comma, "Comma", "Comma");
	keys[Period] = KeyInfo('.', Period, "Period", "Period");
	keys[Quote] = KeyInfo('"', Quote, "Quote", "Quote");
	keys[LCtrl] = KeyInfo(0, LCtrl, "LCtrl", "Left Control");
	keys[RCtrl] = KeyInfo(0, RCtrl, "RCtrl", "Right Control");
	keys[LAlt] = KeyInfo(0, LAlt, "LAlt", "Left Alt");
	keys[RAlt] = KeyInfo(0, RAlt, "RAlt", "Right Alt");
	keys[LShift] = KeyInfo(0, LShift, "LShift", "Left Shift");
	keys[RShift] = KeyInfo(0, RShift, "RShift", "Right Shift");
	keys[LMeta] = KeyInfo(0, LMeta, "LMeta", "Left Meta");
	keys[RMeta] = KeyInfo(0, RMeta, "RMeta", "Right Meta");
	keys[RMenu] = KeyInfo(0, RMenu, "RMenu", "Right Menu");
	keys[Tab] = KeyInfo('\t', Tab, "Tab", "Tab");
	keys[CapsLock] = KeyInfo(0, CapsLock, "CapsLock", "Caps Lock");
	keys[ScrollLock] = KeyInfo(0, ScrollLock, "ScrollLock", "Scroll Lock");
	keys[NumLock] = KeyInfo(0, NumLock, "NumLock", "Num Lock");
	keys[Tilde] = KeyInfo('~', Tilde, "Tilde", "Tilde");
	keys[Numpad0] = KeyInfo('0', Numpad0, "Numpad0", "Numpad 0");
	keys[Numpad1] = KeyInfo('1', Numpad1, "Numpad1", "Numpad 1");
	keys[Numpad2] = KeyInfo('2', Numpad2, "Numpad2", "Numpad 2");
	keys[Numpad3] = KeyInfo('3', Numpad3, "Numpad3", "Numpad 3");
	keys[Numpad4] = KeyInfo('4', Numpad4, "Numpad4", "Numpad 4");
	keys[Numpad5] = KeyInfo('5', Numpad5, "Numpad5", "Numpad 5");
	keys[Numpad6] = KeyInfo('6', Numpad6, "Numpad6", "Numpad 6");
	keys[Numpad7] = KeyInfo('7', Numpad7, "Numpad7", "Numpad 7");
	keys[Numpad8] = KeyInfo('8', Numpad8, "Numpad8", "Numpad 8");
	keys[Numpad9] = KeyInfo('9', Numpad9, "Numpad9", "Numpad 9");
	keys[NumpadDot] = KeyInfo('.', NumpadDot, "NumpadDot", "Numpad .");
	keys[NumpadEnter] = KeyInfo(0xa, NumpadEnter, "NumpadEnter", "Numpad Enter");
	keys[NumpadPlus] = KeyInfo('+', NumpadPlus, "NumpadPlus", "Numpad +");
	keys[NumpadMinus] = KeyInfo('-', NumpadMinus, "NumpadMinus", "Numpad -");
	keys[NumpadAsterisk] = KeyInfo('*', NumpadAsterisk, "NumpadAsterisk", "Numpad *");
	keys[NumpadDivide] = KeyInfo('/', NumpadDivide, "NumpadDiv", "Numpad Divide");
	keys[Decimal] = KeyInfo(0, Decimal, "Decimal");
	keys[Plus] = KeyInfo(0, Plus, "Plus");
	keys[Minus] = KeyInfo(0, Minus, "Minus");
	keys[Insert] = KeyInfo(0, Insert, "Insert", "Insert");
	keys[Home] = KeyInfo(0, Home, "Home", "Home");
	keys[PageUp] = KeyInfo(0, PageUp, "PageUp", "PageUp");
	keys[Delete] = KeyInfo(0, Delete, "Delete", "Delete");
	keys[End] = KeyInfo(0, End, "End", "End");
	keys[PageDown] = KeyInfo(0, PageDown, "PageDown", "PageDown");
	keys[PrintScreen] = KeyInfo(0, PrintScreen, "PrintScreen", "PrintScreen");
	keys[Pause] = KeyInfo(0, Pause, "Pause", "Pause");
	keys[F1] = KeyInfo(0, F1, "F1", "F1");
	keys[F2] = KeyInfo(0, F2, "F2", "F2");
	keys[F3] = KeyInfo(0, F3, "F3", "F3");
	keys[F4] = KeyInfo(0, F4, "F4", "F4");
	keys[F5] = KeyInfo(0, F5, "F5", "F5");
	keys[F6] = KeyInfo(0, F6, "F6", "F6");
	keys[F7] = KeyInfo(0, F7, "F7", "F7");
	keys[F8] = KeyInfo(0, F8, "F8", "F8");
	keys[F9] = KeyInfo(0, F9, "F9", "F9");
	keys[F10] = KeyInfo(0, F10, "F10", "F10");
	keys[F11] = KeyInfo(0, F11, "F11", "F11");
	keys[F12] = KeyInfo(0, F12, "F12", "F12");
	keys[Escape] = KeyInfo(0x1b, Escape, "Escape", "Escape");
	keys[Backspace] = KeyInfo(0x8, Backspace, "Backspace", "Backspace");
	keys[Enter] = KeyInfo(0xa, Enter, "Enter", "Enter");
	keys[Left] = KeyInfo(0, Left, "LeftArrow", "Left Arrow");
	keys[Right] = KeyInfo(0, Right, "RightArrow", "Right Arrow");
	keys[Up] = KeyInfo(0, Up, "UpArrow", "Up Arrow");
	keys[Down] = KeyInfo(0, Down, "DownArrow", "Down Arrow");
	keys[SpaceBar] = KeyInfo(0, SpaceBar, "SpaceBar", "Space Bar");
	keys[Clear] = KeyInfo(0, Clear, "Clear", "Clear");
	keys[Select] = KeyInfo(0, Select, "Select");
	keys[Execute] = KeyInfo(0, Execute, "Execute");
	keys[Print] = KeyInfo(0, Print, "Print");
	keys[Help] = KeyInfo(0, Help, "Help");
	keys[Apps] = KeyInfo(0, Apps, "Apps");
	keys[Sleep] = KeyInfo(0, Sleep, "Sleep");
	keys[BrowserBack] = KeyInfo(0, BrowserBack, "BrowserBack");
	keys[BrowserForward] = KeyInfo(0, BrowserForward, "BrowserForward");
	keys[BrowserRefresh] = KeyInfo(0, BrowserRefresh, "BrowserRefresh");
	keys[BrowserStop] = KeyInfo(0, BrowserStop, "BrowserStop");
	keys[BrowserSearch] = KeyInfo(0, BrowserSearch, "BrowserSearch");
	keys[BrowserFavorites] = KeyInfo(0, BrowserFavorites, "BrowserFavorites");
	keys[BrowserHome] = KeyInfo(0, BrowserHome, "BrowserHome");
	keys[VolumeMute] = KeyInfo(0, VolumeMute, "VolumeMute");
	keys[VolumeDown] = KeyInfo(0, VolumeDown, "VolumeDown");
	keys[VolumeUp] = KeyInfo(0, VolumeUp, "VolumeUp");
	keys[MediaNextTrack] = KeyInfo(0, MediaNextTrack, "MediaNextTrack");
	keys[MediaPrevTrack] = KeyInfo(0, MediaPrevTrack, "MediaPrevTrack");
	keys[MediaStop] = KeyInfo(0, MediaStop, "MediaStop");
	keys[MediaPlayPause] = KeyInfo(0, MediaPlayPause, "MediaPlayPause");
	keys[LaunchMail] = KeyInfo(0, LaunchMail, "LaunchMail");
	keys[LaunchMediaSelect] = KeyInfo(0, LaunchMediaSelect, "LaunchMediaSelect");
	keys[LaunchApp1] = KeyInfo(0, LaunchApp1, "LaunchApp1");
	keys[LaunchApp2] = KeyInfo(0, LaunchApp2, "LaunchApp2");
	
	keys[UnknownKey] = KeyInfo(0, UnknownKey, "unknownkey", "Unknown Key");
	
	qDebug() << "KeyInfo array was initialized";
	assert(assertKeyCodes());
	KeyInfo::bIsInit = true;
}

const KeyInfo& KeyInfo::fromCodeName(const QString& codeName)
{
	if (!KeyInfo::bIsInit)
	{
		initKeys();
	}

	for (int i = 0; i < NUM_KEYS; ++i)
	{
		if (keys[i].codeName().compare(codeName, Qt::CaseInsensitive) == 0)
		{
			return keys[i];
		}
	}
	
	return keys[UnknownKey];
}

const KeyInfo& KeyInfo::fromKeyId(KeyId keyId)
{
	if (!KeyInfo::bIsInit)
	{
		initKeys();
	}

	return keys[keyId];
}
