//------------------------------------------------------------------------------
// keyinfo.h
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#ifndef __KEYINFO_H__
#define __KEYINFO_H__

#include <QString>

class KeyInfo
{
	public:
		enum KeyId
		{
			Key_0,
			Key_1,
			Key_2,
			Key_3,
			Key_4,
			Key_5,
			Key_6,
			Key_7,
			Key_8,
			Key_9,
			Key_A,
			Key_B,
			Key_C,
			Key_D,
			Key_E,
			Key_F,
			Key_G,
			Key_H,
			Key_I,
			Key_J,
			Key_K,
			Key_L,
			Key_M,
			Key_N,
			Key_O,
			Key_P,
			Key_Q,
			Key_R,
			Key_S,
			Key_T,
			Key_V,
			Key_U,
			Key_W,
			Key_X,
			Key_Y,
			Key_Z,
			BackSlash,
			Slash,
			Semicolon,
			LBracket,
			RBracket,
			Comma,
			Period,
			Quote,
			LCtrl,
			RCtrl,
			LAlt,
			RAlt,
			LShift,
			RShift,
			LMeta,
			RMeta,
			RMenu,
			Tab,
			Tilde,
			CapsLock,
			ScrollLock,
			NumLock,
			Numpad0,
			Numpad1,
			Numpad2,
			Numpad3,
			Numpad4,
			Numpad5,
			Numpad6,
			Numpad7,
			Numpad8,
			Numpad9,
			NumpadDot,
			NumpadEnter,
			NumpadPlus,
			NumpadMinus,
			NumpadAsterisk,
			NumpadDivide,
			Decimal,
			Plus,
			Minus,
			Insert,
			Home,
			PageUp,
			Delete,
			End,
			PageDown,
			PrintScreen,
			Pause,
			F1,
			F2,
			F3,
			F4,
			F5,
			F6,
			F7,
			F8,
			F9,
			F10,
			F11,
			F12,
			Escape,
			Backspace,
			Enter,
			Left,
			Right,
			Up,
			Down,
			SpaceBar,
			Clear,
			Select,
			Execute,
			Print,
			Help,
			Apps,
			Sleep,
			BrowserBack,
			BrowserForward,
			BrowserRefresh,
			BrowserStop,
			BrowserSearch,
			BrowserFavorites,
			BrowserHome,
			VolumeMute,
			VolumeDown,
			VolumeUp,
			MediaNextTrack,
			MediaPrevTrack,
			MediaStop,
			MediaPlayPause,
			LaunchMail,
			LaunchMediaSelect,
			LaunchApp1,
			LaunchApp2,
			UnknownKey,
			NUM_KEYS
		};
		
		static bool isShift(KeyId keyId)
		{
			return keyId == LShift || keyId == RShift;
		}
		
		static bool isAlt(KeyId keyId)
		{
			return keyId == LAlt || keyId == RAlt;
		}
		
		static bool isCtrl(KeyId keyId)
		{
			return keyId == LCtrl || keyId == RCtrl;
		}
		
		static bool isMeta(KeyId keyId)
		{
			return keyId == LMeta || keyId == RMeta;
		}
		
		static const KeyInfo& fromCodeName(const QString& codeName);
		static const KeyInfo& fromKeyId(KeyId keyId);
		
		KeyInfo();
		
		int asciiCode() const
		{
			return d.asciiCode;
		}

		const QString& codeName() const
		{
			return d.codeName;
		}
		
		KeyId keyId() const
		{
			return (KeyId) keyNum();
		}
		
		int keyNum() const
		{
			return d.keyNum;
		}
		
		bool isShift() const
		{
			return isShift(keyId());
		}
		
		bool isAlt() const
		{
			return isAlt(keyId());
		}
		
		bool isCtrl() const
		{
			return isCtrl(keyId());
		}
		
		bool isMeta() const
		{
			return isMeta(keyId());
		}
		
		const QString& niceName() const
		{
			return d.niceName;
		}

	private:
		class PrivData
		{
			public:
				int asciiCode;
				QString codeName;
				int keyNum;
				QString niceName;

		};

		static bool bIsInit;
		static KeyInfo keys[NUM_KEYS];
		
		static bool assertKeyCodes();
		static void initKeys();

		PrivData d;

		KeyInfo(int asciiCode, int keyNum, const QString& codeName, const QString& niceName);
		KeyInfo(int asciiCode, int keyNum, const QString& codeName);
};

#endif

