//------------------------------------------------------------------------------
// keystatemonitor.cpp
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "keystatemonitor.h"

#include "keyboard/keystatereader.h"
#include <cassert>
#include <QDebug>

KeyStateMonitor::KeyStateMonitor()
{
	pKeyStateReader = NULL;

	this->connect(&monitorTimer, SIGNAL( timeout() ),
		SLOT( readKeyboardState() ) );
}

QList<KeyInfo::KeyId> KeyStateMonitor::pressedKeys() const
{
	QList<KeyInfo::KeyId> keysPressed = keyStateMap.keys(Pressed);
	return keysPressed;
}

void KeyStateMonitor::readKeyboardState()
{
	QMap<KeyInfo::KeyId, int> keysStates = pKeyStateReader->getCurrentState();
	updateStates(keysStates);
}

bool KeyStateMonitor::resumeMonitoring()
{
	if (this->pKeyStateReader != NULL)
	{
		monitorTimer.start();
		return true;
	}
	
	return false;
}

void KeyStateMonitor::setKeys(QSet<KeyInfo::KeyId> keys)
{
	foreach (KeyInfo::KeyId key, keys)
	{
		keyStateMap.insert(key, Depressed);
	}
}

void KeyStateMonitor::startMonitoring(KeyStateReader* pKeyStateReader, unsigned intervalMs)
{
	assert(pKeyStateReader != NULL);
	assert(intervalMs > 0);

	this->pKeyStateReader = pKeyStateReader;

	monitorTimer.stop();
	monitorTimer.start(intervalMs);
}

const QMap<KeyInfo::KeyId, int>& KeyStateMonitor::state() const
{
	return keyStateMap;
}

void KeyStateMonitor::stopMonitoring()
{
	monitorTimer.stop();
}

void KeyStateMonitor::updateStates(const QMap<KeyInfo::KeyId, int>& newKeyStates)
{
	QMap<KeyInfo::KeyId, int> updatedKeys;
	QList<KeyInfo::KeyId> keys = newKeyStates.keys();
	foreach (KeyInfo::KeyId key, keys)
	{
		if (keyStateMap.contains(key))
		{
			int oldState = keyStateMap[key];
			int newState = newKeyStates[key];
			if (oldState != newState)
			{
				keyStateMap[key] = newState;
				updatedKeys.insert(key, newState);
			}
		}
	}

	keys = updatedKeys.keys();
	foreach (KeyInfo::KeyId key, keys)
	{
		emit keyStateChanged(key, updatedKeys[key]);
	}
	
	emit keyBatchStateChanged(updatedKeys);
}

