//------------------------------------------------------------------------------
// keystatereader.h
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#ifndef __KEYSTATEREADER_H__
#define __KEYSTATEREADER_H__

#include <QMap>
#include <QSet>

#include "keyboard/keyinfo.h"

/**
 * @brief Reads states of all keyboard keys and returns them in
 *        a Qt map.
 *
 * This is an abstract base class for OS specific readers.
 *
 * Keyboard state is supposed to be read using the OS specific api and then
 * translated to the format defined by KeyInfo::KeyId enum. The state values 
 * are defined by KeyStateMonitor::KeyState enum.
 */
class KeyStateReader
{
	public:
		/**
 		* @brief Creates a new KeyStateReader specific to the current
 		*        OS.
 		*
 		* The ownership of the newly created object is returned to the user.
 		* Multiple objects may be created using this method, but they all
 		* must be deleted manually.
 		*/
		static KeyStateReader* instantiateOsSpecificReader();

		/**
		 * @brief All keys that can be extracted from this reader.
		 */
		virtual QSet<KeyInfo::KeyId> allKnownQtKeys() = 0;

		/**
		 * @brief Gets current keyboard state and returns it in a QMap.
		 *
 		* The integer value here must be one of the values defined by
 		* the KeyStateMonitor::KeyState enum.
		 */
		virtual QMap<KeyInfo::KeyId, int> getCurrentState() = 0;

	private:
};

#endif
