//------------------------------------------------------------------------------
// shortcut.h
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#ifndef __SHORTCUT_H__
#define __SHORTCUT_H__

#include <QList>
#include <QObject>

#include "keyboard/keyinfo.h"

/**
 * @brief Describes keyboard shortcuts.
 */
class Shortcut : public QObject
{
	Q_OBJECT

	public:
		Shortcut();
		Shortcut(const QList<KeyInfo::KeyId>& keys);
		
		/**
		 * @brief Check if keys are the same.
		 *
		 * @return true if and only if all keys on the specified list are the same
		 *         as on the list set by setKeys(). 
		 */
		bool areKeysFitting(const QList<KeyInfo::KeyId>& keys) const;
		void fireSignalIfKeysAreFitting(const QList<KeyInfo::KeyId>& keys);

		/**
		 * @brief List of keys associated with this shortcut.
		 */
		const QList<KeyInfo::KeyId>& keys() const;
		
		bool isEnabled() const
		{
			return bEnabled;
		}
		
		/**
		 * @brief Sets the list of keys for which this shortcut will be fired.
		 */
		void setKeys(const QList<KeyInfo::KeyId>& keys);
		
	public slots:
		void setEnabled(bool bEnabled)
		{
			this->bEnabled = bEnabled;
		}
		
	signals:
		void shortcutFired();

	private:
		bool bEnabled;
		QList<KeyInfo::KeyId> keysList;
};

#endif
