//------------------------------------------------------------------------------
// shortcutsmonitor.cpp
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "shortcutsmonitor.h"

#include "keyboard/keystatemonitor.h"
#include "keyboard/shortcut.h"
#include <cassert>
#include <QDebug>

ShortcutsMonitor::ShortcutsMonitor(KeyStateMonitor* pKeyStateMonitor)
{
	assert(pKeyStateMonitor != NULL);
	this->bEnabled = true;
	this->pKeyStateMonitor = pKeyStateMonitor;
	
	this->connect(pKeyStateMonitor, 
		SIGNAL(keyBatchStateChanged(QMap<KeyInfo::KeyId, int>)),
		SLOT  (keyBatchStateChange(QMap<KeyInfo::KeyId, int>)) );
}

void ShortcutsMonitor::fireShortcutsForKeys(const QList<KeyInfo::KeyId>& keys)
{
	foreach (Shortcut* pShortcut, shortcutsList)
	{
		pShortcut->fireSignalIfKeysAreFitting(keys);
	}
}
		
void ShortcutsMonitor::keyBatchStateChange(QMap<KeyInfo::KeyId, int> keyStates)
{
	if (isEnabled())
	{
		// We will detect new shortcuts only if key state changed to pressed.
		QList<KeyInfo::KeyId> newlyPressedKeys = keyStates.keys(KeyStateMonitor::Pressed);
		if (!newlyPressedKeys.isEmpty())
		{
			QList<KeyInfo::KeyId> allPressedKeys = pKeyStateMonitor->pressedKeys();
			fireShortcutsForKeys(allPressedKeys);
		}
	}
}
		
void ShortcutsMonitor::registerShortcut(Shortcut* pShortcut)
{
	if (!shortcutsList.contains(pShortcut))
	{
		shortcutsList << pShortcut;
	}
}

void ShortcutsMonitor::unregisterShortcut(Shortcut* pShortcut)
{
	shortcutsList.removeAll(pShortcut);
}
