//------------------------------------------------------------------------------
// keysdictionary_win.cpp
//
// Notes:
// WINDOWS ONLY!
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "keysdictionary_win.h"

#ifdef Q_WS_WIN
#include <Windows.h>

KeysDictionary_Win* KeysDictionary_Win::pInstance = NULL;

KeysDictionary_Win::KeysDictionary_Win()
{
	initWindowsKeymap();
}

QSet<KeyInfo::KeyId> KeysDictionary_Win::allKnownQtKeys() const
{
	return windowsKeys.values().toSet();
}

KeyInfo::KeyId KeysDictionary_Win::fromNative(int vkCode) const
{
	if (windowsKeys.contains(vkCode))
	{
		return windowsKeys[vkCode];
	}
	else
	{
		return KeyInfo::UnknownKey;
	}
}

void KeysDictionary_Win::initWindowsKeymap()
{
	// Keys presented in the same order as on:
	// http://msdn.microsoft.com/en-us/library/windows/desktop/dd375731%28v=vs.85%29.aspx
	// Note: Some keys may not be supported and were omitted.
	windowsKeys.insert(VK_BACK, KeyInfo::Backspace);
	windowsKeys.insert(VK_TAB, KeyInfo::Tab);
	windowsKeys.insert(VK_CLEAR, KeyInfo::Clear);
	windowsKeys.insert(VK_RETURN, KeyInfo::Enter);
	windowsKeys.insert(VK_PAUSE, KeyInfo::Pause);
	windowsKeys.insert(VK_CAPITAL, KeyInfo::CapsLock);
	windowsKeys.insert(VK_ESCAPE, KeyInfo::Escape);
	windowsKeys.insert(VK_SPACE, KeyInfo::SpaceBar);
	windowsKeys.insert(VK_PRIOR, KeyInfo::PageUp);
	windowsKeys.insert(VK_NEXT, KeyInfo::PageDown);
	windowsKeys.insert(VK_END, KeyInfo::End);
	windowsKeys.insert(VK_HOME, KeyInfo::Home);
	windowsKeys.insert(VK_LEFT, KeyInfo::Left);
	windowsKeys.insert(VK_UP, KeyInfo::Up);
	windowsKeys.insert(VK_RIGHT, KeyInfo::Right);
	windowsKeys.insert(VK_DOWN, KeyInfo::Down);
	windowsKeys.insert(VK_SELECT, KeyInfo::Select);
	windowsKeys.insert(VK_PRINT, KeyInfo::Print);
	windowsKeys.insert(VK_EXECUTE, KeyInfo::Execute);
	windowsKeys.insert(VK_SNAPSHOT, KeyInfo::PrintScreen);
	windowsKeys.insert(VK_INSERT, KeyInfo::Insert);
	windowsKeys.insert(VK_DELETE, KeyInfo::Delete);
	windowsKeys.insert(VK_HELP, KeyInfo::Help);

	// 0 - 9
	for (int key = KeyInfo::Key_0; key <= KeyInfo::Key_9; ++key)
	{
		int offset = key - KeyInfo::Key_0;
		windowsKeys.insert('0' + offset, (KeyInfo::KeyId)key);
	}
	
	// A - Z
	for (int key = KeyInfo::Key_A; key <= KeyInfo::Key_Z; ++key)
	{
		int offset = key - KeyInfo::Key_A;
		windowsKeys.insert('A' + offset, (KeyInfo::KeyId)key);
	}

	windowsKeys.insert(VK_LWIN, KeyInfo::LMeta);
	windowsKeys.insert(VK_RWIN, KeyInfo::RMeta);
	windowsKeys.insert(VK_APPS, KeyInfo::Apps);
	windowsKeys.insert(VK_SLEEP, KeyInfo::Sleep);
	
	// Numpad0 - Numpad9
	for (int key = KeyInfo::Numpad0; key <= KeyInfo::Numpad9; ++key)
	{
		int offset = key - KeyInfo::Numpad0;
		windowsKeys.insert(VK_NUMPAD0 + offset, (KeyInfo::KeyId)key);
	}

	windowsKeys.insert(VK_MULTIPLY, KeyInfo::NumpadAsterisk);
	windowsKeys.insert(VK_ADD, KeyInfo::NumpadPlus);
	windowsKeys.insert(VK_SEPARATOR, KeyInfo::NumpadDot);
	windowsKeys.insert(VK_SUBTRACT, KeyInfo::NumpadMinus);
	windowsKeys.insert(VK_DECIMAL, KeyInfo::Decimal);
	windowsKeys.insert(VK_DIVIDE, KeyInfo::NumpadDivide);
	windowsKeys.insert(VK_F1, KeyInfo::F1);
	windowsKeys.insert(VK_F2, KeyInfo::F2);
	windowsKeys.insert(VK_F3, KeyInfo::F3);
	windowsKeys.insert(VK_F4, KeyInfo::F4);
	windowsKeys.insert(VK_F5, KeyInfo::F5);
	windowsKeys.insert(VK_F6, KeyInfo::F6);
	windowsKeys.insert(VK_F7, KeyInfo::F7);
	windowsKeys.insert(VK_F8, KeyInfo::F8);
	windowsKeys.insert(VK_F9, KeyInfo::F9);
	windowsKeys.insert(VK_F10, KeyInfo::F10);
	windowsKeys.insert(VK_F11, KeyInfo::F11);
	windowsKeys.insert(VK_F12, KeyInfo::F12);
	//windowsKeys.insert(VK_F13, Qt::Key_F13);
	//windowsKeys.insert(VK_F14, Qt::Key_F14);
	//windowsKeys.insert(VK_F15, Qt::Key_F15);
	//windowsKeys.insert(VK_F16, Qt::Key_F16);
	//windowsKeys.insert(VK_F17, Qt::Key_F17);
	//windowsKeys.insert(VK_F18, Qt::Key_F18);
	//windowsKeys.insert(VK_F19, Qt::Key_F19);
	//windowsKeys.insert(VK_F20, Qt::Key_F20);
	//windowsKeys.insert(VK_F21, Qt::Key_F21);
	//windowsKeys.insert(VK_F22, Qt::Key_F22);
	//windowsKeys.insert(VK_F23, Qt::Key_F23);
	//windowsKeys.insert(VK_F24, Qt::Key_F24);
	windowsKeys.insert(VK_NUMLOCK, KeyInfo::NumLock);
	windowsKeys.insert(VK_SCROLL, KeyInfo::ScrollLock);
	windowsKeys.insert(VK_LSHIFT, KeyInfo::LShift);
	windowsKeys.insert(VK_RSHIFT, KeyInfo::RShift);
	windowsKeys.insert(VK_LCONTROL, KeyInfo::LCtrl);
	windowsKeys.insert(VK_RCONTROL, KeyInfo::RCtrl);
	windowsKeys.insert(VK_LMENU, KeyInfo::LAlt);
	windowsKeys.insert(VK_RMENU, KeyInfo::RAlt);
	windowsKeys.insert(VK_BROWSER_BACK, KeyInfo::BrowserBack);
	windowsKeys.insert(VK_BROWSER_FORWARD, KeyInfo::BrowserForward);
	windowsKeys.insert(VK_BROWSER_REFRESH, KeyInfo::BrowserRefresh);
	windowsKeys.insert(VK_BROWSER_STOP, KeyInfo::BrowserStop);
	windowsKeys.insert(VK_BROWSER_SEARCH, KeyInfo::BrowserSearch);
	windowsKeys.insert(VK_BROWSER_FAVORITES, KeyInfo::BrowserFavorites);
	windowsKeys.insert(VK_BROWSER_HOME, KeyInfo::BrowserHome);
	windowsKeys.insert(VK_VOLUME_MUTE, KeyInfo::VolumeMute);
	windowsKeys.insert(VK_VOLUME_DOWN, KeyInfo::VolumeDown);
	windowsKeys.insert(VK_VOLUME_UP, KeyInfo::VolumeUp);
	windowsKeys.insert(VK_MEDIA_NEXT_TRACK, KeyInfo::MediaNextTrack);
	windowsKeys.insert(VK_MEDIA_PREV_TRACK, KeyInfo::MediaPrevTrack);
	windowsKeys.insert(VK_MEDIA_STOP, KeyInfo::MediaStop);
	windowsKeys.insert(VK_MEDIA_PLAY_PAUSE, KeyInfo::MediaPlayPause);
	windowsKeys.insert(VK_LAUNCH_MAIL, KeyInfo::LaunchMail);
	windowsKeys.insert(VK_LAUNCH_MEDIA_SELECT, KeyInfo::LaunchMediaSelect);
	windowsKeys.insert(VK_LAUNCH_APP1, KeyInfo::LaunchApp1);
	windowsKeys.insert(VK_LAUNCH_APP2, KeyInfo::LaunchApp2);
	windowsKeys.insert(VK_OEM_PLUS, KeyInfo::Plus);
	windowsKeys.insert(VK_OEM_COMMA, KeyInfo::Comma);
	windowsKeys.insert(VK_OEM_MINUS, KeyInfo::Minus);
	windowsKeys.insert(VK_OEM_PERIOD, KeyInfo::Period);
	windowsKeys.insert(VK_OEM_1, KeyInfo::Semicolon);
	windowsKeys.insert(VK_OEM_2, KeyInfo::Slash);
	windowsKeys.insert(VK_OEM_3, KeyInfo::Tilde);
	windowsKeys.insert(VK_OEM_4, KeyInfo::LBracket);
	windowsKeys.insert(VK_OEM_5, KeyInfo::BackSlash);
	windowsKeys.insert(VK_OEM_6, KeyInfo::RBracket);
	windowsKeys.insert(VK_OEM_7, KeyInfo::Quote);
}

#endif
