//------------------------------------------------------------------------------
// keysdictionary_x11.cpp
//
// Notes:
// X11 (Linux) ONLY!
//
// Copyright (C) 2012 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "keysdictionary_x11.h"

#ifdef Q_WS_X11
#include <X11/XF86keysym.h>
#include <X11/keysym.h>

KeysDictionary_X11* KeysDictionary_X11::pInstance = NULL;

KeysDictionary_X11::KeysDictionary_X11()
{
	initKeymap();
}

QSet<KeyInfo::KeyId> KeysDictionary_X11::allKnownQtKeys() const
{
	return nativeKeys.values().toSet();
}

const QList<int>& KeysDictionary_X11::allKnownNativeKeySyms() const
{
	if (nativeKeysList.isEmpty())
	{
		nativeKeysList = nativeKeys.keys();
	}
	return nativeKeysList;
}

KeyInfo::KeyId KeysDictionary_X11::fromNative(int vkCode) const
{
	if (nativeKeys.contains(vkCode))
	{
		return nativeKeys[vkCode];
	}
	else
	{
		return KeyInfo::UnknownKey;
	}
}

void KeysDictionary_X11::initKeymap()
{
	nativeKeys.insert(XK_BackSpace, KeyInfo::Backspace);
	nativeKeys.insert(XK_Tab, KeyInfo::Tab);
	nativeKeys.insert(XK_Clear, KeyInfo::Clear);
	nativeKeys.insert(XK_Return, KeyInfo::Enter);
	nativeKeys.insert(XK_Pause, KeyInfo::Pause);
	nativeKeys.insert(XK_Caps_Lock, KeyInfo::CapsLock);
	nativeKeys.insert(XK_Escape, KeyInfo::Escape);
	nativeKeys.insert(XK_space, KeyInfo::SpaceBar);
	nativeKeys.insert(XK_Page_Up, KeyInfo::PageUp);
	nativeKeys.insert(XK_Page_Down, KeyInfo::PageDown);
	nativeKeys.insert(XK_End, KeyInfo::End);
	nativeKeys.insert(XK_Home, KeyInfo::Home);
	nativeKeys.insert(XK_Left, KeyInfo::Left);
	nativeKeys.insert(XK_Up, KeyInfo::Up);
	nativeKeys.insert(XK_Right, KeyInfo::Right);
	nativeKeys.insert(XK_Down, KeyInfo::Down);
	nativeKeys.insert(XK_Select, KeyInfo::Select);
//	nativeKeys.insert(VK_PRINT, KeyInfo::Print);
	nativeKeys.insert(XK_Execute, KeyInfo::Execute);
	nativeKeys.insert(XK_Print, KeyInfo::PrintScreen);
	nativeKeys.insert(XK_Insert, KeyInfo::Insert);
	nativeKeys.insert(XK_Delete, KeyInfo::Delete);
	nativeKeys.insert(XK_Help, KeyInfo::Help);

	// 0 - 9
	for (int key = KeyInfo::Key_0; key <= KeyInfo::Key_9; ++key)
	{
		int offset = key - KeyInfo::Key_0;
		nativeKeys.insert(XK_0 + offset, (KeyInfo::KeyId)key);
	}

	// A - Z
	for (int key = KeyInfo::Key_A; key <= KeyInfo::Key_Z; ++key)
	{
		int offset = key - KeyInfo::Key_A;
		nativeKeys.insert(XK_a + offset, (KeyInfo::KeyId)key);
	}

	nativeKeys.insert(XK_Super_L, KeyInfo::LMeta);
	nativeKeys.insert(XK_Super_R, KeyInfo::RMeta);

	// Numpad0 - Numpad9
	for (int key = KeyInfo::Numpad0; key <= KeyInfo::Numpad9; ++key)
	{
		int offset = key - KeyInfo::Numpad0;
		nativeKeys.insert(XK_KP_0 + offset, (KeyInfo::KeyId)key);
	}

	nativeKeys.insert(XK_KP_Multiply, KeyInfo::NumpadAsterisk);
	nativeKeys.insert(XK_KP_Add, KeyInfo::NumpadPlus);
	nativeKeys.insert(XK_KP_Separator, KeyInfo::NumpadDot);
	nativeKeys.insert(XK_KP_Subtract, KeyInfo::NumpadMinus);
	nativeKeys.insert(XK_KP_Decimal, KeyInfo::Decimal);
	nativeKeys.insert(XK_KP_Divide, KeyInfo::NumpadDivide);
	nativeKeys.insert(XK_KP_Enter, KeyInfo::NumpadEnter);
	nativeKeys.insert(XK_F1, KeyInfo::F1);
	nativeKeys.insert(XK_F2, KeyInfo::F2);
	nativeKeys.insert(XK_F3, KeyInfo::F3);
	nativeKeys.insert(XK_F4, KeyInfo::F4);
	nativeKeys.insert(XK_F5, KeyInfo::F5);
	nativeKeys.insert(XK_F6, KeyInfo::F6);
	nativeKeys.insert(XK_F7, KeyInfo::F7);
	nativeKeys.insert(XK_F8, KeyInfo::F8);
	nativeKeys.insert(XK_F9, KeyInfo::F9);
	nativeKeys.insert(XK_F10, KeyInfo::F10);
	nativeKeys.insert(XK_F11, KeyInfo::F11);
	nativeKeys.insert(XK_F12, KeyInfo::F12);
//	//nativeKeys.insert(XK_F13, Qt::Key_F13);
//	//nativeKeys.insert(XK_F14, Qt::Key_F14);
//	//nativeKeys.insert(XK_F15, Qt::Key_F15);
//	//nativeKeys.insert(XK_F16, Qt::Key_F16);
//	//nativeKeys.insert(XK_F17, Qt::Key_F17);
//	//nativeKeys.insert(XK_F18, Qt::Key_F18);
//	//nativeKeys.insert(XK_F19, Qt::Key_F19);
//	//nativeKeys.insert(XK_F20, Qt::Key_F20);
//	//nativeKeys.insert(XK_F21, Qt::Key_F21);
//	//nativeKeys.insert(XK_F22, Qt::Key_F22);
//	//nativeKeys.insert(XK_F23, Qt::Key_F23);
//	//nativeKeys.insert(XK_F24, Qt::Key_F24);
	nativeKeys.insert(XK_Num_Lock, KeyInfo::NumLock);
	nativeKeys.insert(XK_Scroll_Lock, KeyInfo::ScrollLock);
	nativeKeys.insert(XK_Shift_L, KeyInfo::LShift);
	nativeKeys.insert(XK_Shift_R, KeyInfo::RShift);
	nativeKeys.insert(XK_Control_L, KeyInfo::LCtrl);
	nativeKeys.insert(XK_Control_R, KeyInfo::RCtrl);
	nativeKeys.insert(XK_Alt_L, KeyInfo::LAlt);
	nativeKeys.insert(XK_Alt_R, KeyInfo::RAlt);
	nativeKeys.insert(XF86XK_Back, KeyInfo::BrowserBack);
	nativeKeys.insert(XF86XK_Forward, KeyInfo::BrowserForward);
	nativeKeys.insert(XF86XK_Refresh, KeyInfo::BrowserRefresh);
	nativeKeys.insert(XK_Cancel, KeyInfo::BrowserStop);
	nativeKeys.insert(XF86XK_Search, KeyInfo::BrowserSearch);
	nativeKeys.insert(XF86XK_Favorites, KeyInfo::BrowserFavorites);
	nativeKeys.insert(XF86XK_HomePage, KeyInfo::BrowserHome);
	nativeKeys.insert(XF86XK_AudioMute, KeyInfo::VolumeMute);
	nativeKeys.insert(XF86XK_AudioLowerVolume, KeyInfo::VolumeDown);
	nativeKeys.insert(XF86XK_AudioRaiseVolume, KeyInfo::VolumeUp);
	nativeKeys.insert(XF86XK_AudioNext, KeyInfo::MediaNextTrack);
	nativeKeys.insert(XF86XK_AudioPrev, KeyInfo::MediaPrevTrack);
	nativeKeys.insert(XF86XK_AudioStop, KeyInfo::MediaStop);
	nativeKeys.insert(XF86XK_AudioPlay, KeyInfo::MediaPlayPause);
	nativeKeys.insert(XF86XK_Mail, KeyInfo::LaunchMail);
	nativeKeys.insert(XF86XK_AudioMedia, KeyInfo::LaunchMediaSelect);
//	nativeKeys.insert(VK_LAUNCH_APP1, KeyInfo::LaunchApp1);
//	nativeKeys.insert(VK_LAUNCH_APP2, KeyInfo::LaunchApp2);
	nativeKeys.insert(XK_equal, KeyInfo::Plus);
	nativeKeys.insert(XK_comma, KeyInfo::Comma);
	nativeKeys.insert(XK_minus, KeyInfo::Minus);
	nativeKeys.insert(XK_period, KeyInfo::Period);
	nativeKeys.insert(XK_semicolon, KeyInfo::Semicolon);
	nativeKeys.insert(XK_slash, KeyInfo::Slash);
	nativeKeys.insert(XK_grave, KeyInfo::Tilde);
	nativeKeys.insert(XK_bracketleft, KeyInfo::LBracket);
	nativeKeys.insert(XK_backslash, KeyInfo::BackSlash);
	nativeKeys.insert(XK_bracketright, KeyInfo::RBracket);
	nativeKeys.insert(XK_apostrophe, KeyInfo::Quote);
}

#endif
