//------------------------------------------------------------------------------
// keystatereader_x11.cpp
//
// Notes:
// X11 (Linux) ONLY!
//
// Copyright (C) 2012 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "keystatereader_x11.h"

#ifdef Q_WS_X11
#include "keyboard/x11/keysdictionary_x11.h"
#include "keyboard/keystatemonitor.h"
#include <iostream>
#include <QDebug>
#include <QX11Info>
#include <X11/keysym.h>
#include <X11/Xlib.h>
#include <X11/XKBlib.h>

QSet<KeyInfo::KeyId> KeyStateReader_X11::allKnownQtKeys()
{
	return KeysDictionary_X11::get()->allKnownQtKeys();
}

QMap<KeyInfo::KeyId, int> KeyStateReader_X11::getCurrentState()
{
	const int MIN_KEYSYM = 8;
	const int MAX_KEYSYM = 256;
	const int KEYMAP_BUFFER_SIZE = 32;

	QMap<KeyInfo::KeyId, int> map;
	char keymapBuffer[KEYMAP_BUFFER_SIZE];
	memset(keymapBuffer, 0, sizeof(KEYMAP_BUFFER_SIZE));

	Display* pDisplay = QX11Info::display();

	// Get the state of all keyboard keys.
	XQueryKeymap(pDisplay, keymapBuffer);

//  // DEBUG CODE. Please do not remove :)
//	for (int i = 0; i < KEYMAP_BUFFER_SIZE; ++i)
//	{
//		std::cout.setf ( std::ios::hex, std::ios::basefield );
//		std::cout.setf ( std::ios::showbase );
//
//		if (keymapBuffer[i] != 0)
//		{
//			for (int offsetIdx = 0; offsetIdx < 8; ++offsetIdx)
//			{
//				int offset = 1 << offsetIdx;
//				if ((offset & keymapBuffer[i]) != 0)
//				{
//					char none[] = "NULL";
//					int keyCode = i * 8 + offsetIdx;
//					int keySym = XkbKeycodeToKeysym(pDisplay, keyCode, 0, 0);
//					char* str = XKeysymToString(keySym);
//					if (str == NULL)  str = none;
//					std::cout << (int)keymapBuffer[i] << " " << keyCode << " " << keySym << " " << str << std::endl;
//				}
//			}
//		}
//	}
//	std::cout << "|||" << std::endl << std::endl;

	QList<int> keySyms = KeysDictionary_X11::get()->allKnownNativeKeySyms();
	foreach (int nativeKeySym, keySyms)
	{
		KeyInfo::KeyId key = KeysDictionary_X11::get()->fromNative(nativeKeySym);
		if (key != KeyInfo::UnknownKey)
		{
			KeyCode nativeKeyCode = XKeysymToKeycode(pDisplay, nativeKeySym);

			// Explanation:
			// XQueryKeymap returns a bit buffer of 32 * 8 bits.
			// The 'nativeKeyCode' here is actually an address of that bit in
			// the key map. We calculate the proper address and extract
			// the key state.
			int keymapByteOffset = nativeKeyCode >> 3; // Divide by 8
			int keymapBitOffset = 1 << (nativeKeyCode % 8);

			int state = keymapBuffer[keymapByteOffset] & keymapBitOffset;

			KeyStateMonitor::KeyState keyState;
			keyState = (state != 0)
				? KeyStateMonitor::Pressed
				: KeyStateMonitor::Depressed;

			map.insert(key, keyState);
		}
	}

	return map;
}
#endif
