//------------------------------------------------------------------------------
// paths.cpp
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#include "paths.h"

#include <QDateTime>
#include <QDesktopServices>
#include <QFileInfo>
#include "versioninfo.h"

QString Paths::documentsPath()
{
	QString localData = QDesktopServices::storageLocation(QDesktopServices::DocumentsLocation);
	localData += "/" + VersionInfo::appDataDirName() + "/";

	return QFileInfo(localData).absoluteFilePath();
}

QString Paths::ensureUniquePath(const QString& path)
{
	QFileInfo fi(path);
	
	// Obtain all necessary information for future path generation.
	QString completeBaseName = fi.completeBaseName();
	QString suffix = fi.suffix();
	QString dirPath = fi.path();
	
	int i = 1;
	while (fi.exists())
	{
		QString path = "%1/%2-%3.%4";
		path.arg(dirPath).arg(completeBaseName).arg(i).arg(suffix);
		fi.setFile(path);
		++i;
	}
	
	return fi.filePath();
}

QString Paths::resolveOutputFilePath(const QString& pathTemplate)
{
	qint64 timestamp = QDateTime::currentMSecsSinceEpoch();
	
	QString resolvedPath = pathTemplate;
	resolvedPath.replace("$TIMESTAMP", QString("%1").arg(timestamp));
	return resolvedPath;
}
