//------------------------------------------------------------------------------
// paths.h
//
// Copyright (C) 2011 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
#ifndef __PATHS_H__
#define __PATHS_H__

#include <QString>

class Paths
{
	public:
		static QString documentsPath();
		
		/**
		 * @brief Ensures that the given path is unique within the filesystem.
		 *
		 * If path is unique then no change is made.
		 * If an object already exists under specified path, the path
		 * will be expanded with an incrementing number that will be
		 * as high as required to find unique path.
		 *
		 * Warning: this may be not very efficient if there are
		 * thousands of numbered files in the same directory.
		 *
		 * File extension is preserved.
		 *
		 * @return Unique path within the filesystem.
		 */		
		static QString ensureUniquePath(const QString& path);
		
		/**
		 * @brief Takes the output path template and spits out a valid,
		 *        ready to use path.
		 *
		 * The template may contain following tags:
		 * - $TIMESTAMP - replaced with current system time in milliseconds.
		 *
		 * The path might not be unique within the filesystem. To ensure
		 * that, call ensureUniquePath. 
		 */
		static QString resolveOutputFilePath(const QString& pathTemplate);
};

#endif